@@ -35,193 +35,220 @@
 #import "MEGASet.h"
 #import "MEGASetElement.h"
 #import "MEGAVPNCredentials.h"
+#import "MEGANetworkConnectivityTestResults.h"
 #import "MEGAVPNRegion.h"
 #import "MEGANotificationList.h"
 
 NS_ASSUME_NONNULL_BEGIN
 
 typedef NS_ENUM (NSInteger, MEGARequestType) {
-    MEGARequestTypeLogin,
-    MEGARequestTypeCreateFolder,
-    MEGARequestTypeMove,
-    MEGARequestTypeCopy,
-    MEGARequestTypeRename,
-    MEGARequestTypeRemove,
-    MEGARequestTypeShare,
-    MEGARequestTypeImportLink,
-    MEGARequestTypeExport,
-    MEGARequestTypeFetchNodes,
-    MEGARequestTypeAccountDetails,
-    MEGARequestTypeChangePassword,
-    MEGARequestTypeUpload,
-    MEGARequestTypeLogout,
-    MEGARequestTypeGetPublicNode,
-    MEGARequestTypeGetAttrFile,
-    MEGARequestTypeSetAttrFile,
-    MEGARequestTypeGetAttrUser,
-    MEGARequestTypeSetAttrUser,
-    MEGARequestTypeRetryPendingConnections,
-    MEGARequestTypeRemoveContact,
-    MEGARequestTypeCreateAccount,
-    MEGARequestTypeConfirmAccount,
-    MEGARequestTypeQuerySignUpLink,
-    MEGARequestTypeAddSync,
-    MEGARequestTypeRemoveSync,
-    MEGARequestTypeDisableSync,
-    MEGARequestTypeEnableSync,
-    MEGARequestTypeCopySyncConfig,
-    MEGARequestTypeCopyCachedConfig,
-    MEGARequestTypeImportSyncConfigs,
-    MEGARequestTypeRemoveSyncs,
-    MEGARequestTypePauseTransfers,
-    MEGARequestTypeCancelTransfer,
-    MEGARequestTypeCancelTransfers,
-    MEGARequestTypeDelete,
-    MEGARequestTypeReportEvent,
-    MEGARequestTypeCancelAttrFile,
-    MEGARequestTypeGetPricing,
-    MEGARequestTypeGetPaymentId,
-    MEGARequestTypeGetUserData,
-    MEGARequestTypeLoadBalancing,
-    MEGARequestTypeKillSession,
-    MEGARequestTypeSubmitPurchaseReceipt,
-    MEGARequestTypeCreditCardStore,
-    MEGARequestTypeUpgradeAccount,
-    MEGARequestTypeCreditCardQuerySubscriptions,
-    MEGARequestTypeCreditCardCancelSubscriptions,
-    MEGARequestTypeGetSessionTransferUrl,
-    MEGARequestTypeGetPaymentMethods,
-    MEGARequestTypeInviteContact,
-    MEGARequestTypeReplyContactRequest,
-    MEGARequestTypeSubmitFeedback,
-    MEGARequestTypeSendEvent,
-    MEGARequestTypeCleanRubbishBin,
-    MEGARequestTypeSetAttrNode,
-    MEGARequestTypeChatCreate,
-    MEGARequestTypeChatFetch,
-    MEGARequestTypeChatInvite,
-    MEGARequestTypeChatRemove,
-    MEGARequestTypeChatUrl,
-    MEGARequestTypeChatGrantAccess,
-    MEGARequestTypeChatRemoveAccess,
-    MEGARequestTypeUseHttpsOnly,
-    MEGARequestTypeSetProxy,
-    MEGARequestTypeGetRecoveryLink,
-    MEGARequestTypeQueryRecoveryLink,
-    MEGARequestTypeConfirmRecoveryLink,
-    MEGARequestTypeGetCancelLink,
-    MEGARequestTypeConfirmCancelLink,
-    MEGARequestTypeGetChangeEmailLink,
-    MEGARequestTypeConfirmChangeEmailLink,
-    MEGARequestTypeChatUpdatePermissions,
-    MEGARequestTypeChatTruncate,
-    MEGARequestTypeChatSetTitle,
-    MEGARequestTypeSetMaxConnections,
-    MEGARequestTypePauseTransfer,
-    MEGARequestTypeMoveTransfer,
-    MEGARequestTypeChatPresenceUrl,
-    MEGARequestTypeRegisterPushNotification,
-    MEGARequestTypeGetUserEmail,
-    MEGARequestTypeAppVersion,
-    MEGARequestTypeGetLocalSSLCertificate,
-    MEGARequestTypeSendSignupLink,
-    MEGARequestTypeQueryDns,
-    MEGARequestTypeQueryGelb,
-    MEGARequestTypeChatStats,
-    MEGARequestTypeDownloadFile,
-    MEGARequestTypeQueryTransferQuota,
-    MEGARequestTypePasswordLink,
-    MEGARequestTypeGetAchievements,
-    MEGARequestTypeRestore,
-    MEGARequestTypeRemoveVersions,
-    MEGARequestTypeChatArchive,
-    MEGARequestTypeWhyAmIBlocked,
-    MEGARequestTypeContactLinkCreate,
-    MEGARequestTypeContactLinkQuery,
-    MEGARequestTypeContactLinkDelete,
-    MEGARequestTypeFolderInfo,
-    MEGARequestTypeRichLink,
-    MEGARequestTypeKeepMeAlive,
-    MEGARequestTypeMultiFactorAuthCheck,
-    MEGARequestTypeMultiFactorAuthGet,
-    MEGARequestTypeMultiFactorAuthSet,
-    MEGARequestTypeAddBackup,
-    MEGARequestTypeRemoveBackup,
-    MEGARequestTypeTimer,
-    MEGARequestTypeAbortCurrentBackup,
-    MEGARequestTypeGetPSA,
-    MEGARequestTypeFetchTimeZone,
-    MEGARequestTypeUseralertAcknowledge,
-    MEGARequestTypeChatLinkHandle,
-    MEGARequestTypeChatLinkUrl,
-    MEGARequestTypeSetPrivateMode,
-    MEGARequestTypeAutojoinPublicChat,
-    MEGARequestTypeCatchup,
-    MEGARequestTypePublicLinkInformation,
-    MEGARequestTypeGetBackgroundUploadURL,
-    MEGARequestTypeCompleteBackgroundUpload,
-    MEGARequestTypeCloudStorageUsed,
-    MEGARequestTypeSendSMSVerificationCode,
-    MEGARequestTypeCheckSMSVerificationCode,
-    MEGARequestTypeGetRegisteredContacts,  // Deprecated
-    MEGARequestTypeGetCountryCallingCodes,
-    MEGARequestTypeVerifyCredentials,
-    MEGARequestTypeGetMiscFlags,
-    MEGARequestTypeResendVerificationEmail,
-    MEGARequestTypeSupportTicket,
-    MEGARequestTypeRetentionTime,
-    MEGARequestTypeResetSmsVerifiedNumber,
-    MEGARequestTypeSendDevCommand,
-    MEGARequestTypeGetBanners,
-    MEGARequestTypeDismissBanner,
-    MEGARequestTypeBackupPut,
-    MEGARequestTypeBackupRemove,
-    MEGARequestTypeBackupPutHeartbeat,
-    MEGARequestTypeFetchAds,
-    MEGARequestTypeQueryAds,
-    MEGARequestTypeGetAttrNode,
-    MEGARequestTypeLoadExternalDriveBackups,
-    MEGARequestTypeCloseExternalDriveBackups,
-    MEGARequestTypeGetDownloadUrls,
-    MEGARequestTypeStartChatCall,
-    MEGARequestTypeJoinChatCall,
-    MEGARequestTypeEndChatCall,
-    MEGARequestTypeGetFAUploadUrl,
-    MEGARequestTypeExecuteOnThread,
-    MEGARequestTypeGetChatOptions,
-    MEGARequestTypeGetRecentActions,
-    MEGARequestTypeCheckRecoveryKey,
-    MEGARequestTypeSetMyBackups,
-    MEGARequestTypePutSet,
-    MEGARequestTypeRemoveSet,
-    MEGARequestTypeFetchSet,
-    MEGARequestTypePutSetElement,
-    MEGARequestTypeRemoveSetElement,
-    MEGARequestTypeRemoveOldBackupNodes,
-    MEGARequestTypeSetSyncRunstate,
-    MEGARequestTypeAddUpdateScheduledMeeting,
-    MEGARequestTypeDelScheduledMeeting,
-    MEGARequestTypeFetchScheduledMeeting,
-    MEGARequestTypeFetchScheduledMeetingOccurrences,
-    MEGARequestTypeOpenShareDialog,
-    MEGARequestTypeUpgradeSecurity,
-    MEGARequestTypePutSetElements,
-    MEGARequestTypeRemoveSetElements,
-    MEGARequestTypeExportSet,
-    MEGARequestTypeExportedSetElement,
-    MEGARequestTypeGetRecommenedProPlan,
-    MEGARequestTypeBackupInfo,
-    MEGARequestTypeBackupRemoveMD,
-    MEGARequestTypeABTestActive,
-    MEGARequestTypeGetVPNRegions,
-    MEGARequestTypeGetVPNCredentials,
-    MEGARequestTypePutVPNCredentials,
-    MEGARequestTypeDeleteVPNCredentials,
-    MEGARequestTypeCheckVPNCredentials,
-    MEGARequestTypeGetNotifications,
-    MEGARequestTypeNodeTag = 186,
+    MEGARequestTypeLogin = 0,
+    MEGARequestTypeCreateFolder = 1,
+    MEGARequestTypeMove = 2,
+    MEGARequestTypeCopy = 3,
+    MEGARequestTypeRename = 4,
+    MEGARequestTypeRemove = 5,
+    MEGARequestTypeShare = 6,
+    MEGARequestTypeImportLink = 7,
+    MEGARequestTypeExport = 8,
+    MEGARequestTypeFetchNodes = 9,
+    MEGARequestTypeAccountDetails = 10,
+    MEGARequestTypeChangePassword = 11,
+    MEGARequestTypeUpload = 12,
+    MEGARequestTypeLogout = 13,
+    MEGARequestTypeGetPublicNode = 14,
+    MEGARequestTypeGetAttrFile = 15,
+    MEGARequestTypeSetAttrFile = 16,
+    MEGARequestTypeGetAttrUser = 17,
+    MEGARequestTypeSetAttrUser = 18,
+    MEGARequestTypeRetryPendingConnections = 19,
+    MEGARequestTypeRemoveContact = 20,
+    MEGARequestTypeCreateAccount = 21,
+    MEGARequestTypeConfirmAccount = 22,
+    MEGARequestTypeQuerySignUpLink = 23,
+    MEGARequestTypeAddSync = 24,
+    MEGARequestTypeRemoveSync = 25,
+    MEGARequestTypeDisableSync = 26, // Deprecated
+    MEGARequestTypeEnableSync = 27, // Deprecated
+    MEGARequestTypeCopySyncConfig = 28,
+    MEGARequestTypeCopyCachedConfig = 29,
+    MEGARequestTypeImportSyncConfigs = 30,
+    MEGARequestTypeRemoveSyncs = 31,
+    MEGARequestTypePauseTransfers = 32,
+    MEGARequestTypeCancelTransfer = 33,
+    MEGARequestTypeCancelTransfers = 34,
+    MEGARequestTypeDelete = 35,
+    MEGARequestTypeReportEvent = 36,
+    MEGARequestTypeCancelAttrFile = 37,
+    MEGARequestTypeGetPricing = 38,
+    MEGARequestTypeGetPaymentId = 39,
+    MEGARequestTypeGetUserData = 40,
+    MEGARequestTypeLoadBalancing = 41,
+    MEGARequestTypeKillSession = 42,
+    MEGARequestTypeSubmitPurchaseReceipt = 43,
+    MEGARequestTypeCreditCardStore = 44,
+    MEGARequestTypeUpgradeAccount = 45,
+    MEGARequestTypeCreditCardQuerySubscriptions = 46,
+    MEGARequestTypeCreditCardCancelSubscriptions = 47,
+    MEGARequestTypeGetSessionTransferUrl = 48,
+    MEGARequestTypeGetPaymentMethods = 49,
+    MEGARequestTypeInviteContact = 50,
+    MEGARequestTypeReplyContactRequest = 51,
+    MEGARequestTypeSubmitFeedback = 52,
+    MEGARequestTypeSendEvent = 53,
+    MEGARequestTypeCleanRubbishBin = 54,
+    MEGARequestTypeSetAttrNode = 55,
+    MEGARequestTypeChatCreate = 56,
+    MEGARequestTypeChatFetch = 57,
+    MEGARequestTypeChatInvite = 58,
+    MEGARequestTypeChatRemove = 59,
+    MEGARequestTypeChatUrl = 60,
+    MEGARequestTypeChatGrantAccess = 61,
+    MEGARequestTypeChatRemoveAccess = 62,
+    MEGARequestTypeUseHttpsOnly = 63,
+    MEGARequestTypeSetProxy = 64,
+    MEGARequestTypeGetRecoveryLink = 65,
+    MEGARequestTypeQueryRecoveryLink = 66,
+    MEGARequestTypeConfirmRecoveryLink = 67,
+    MEGARequestTypeGetCancelLink = 68,
+    MEGARequestTypeConfirmCancelLink = 69,
+    MEGARequestTypeGetChangeEmailLink = 70,
+    MEGARequestTypeConfirmChangeEmailLink = 71,
+    MEGARequestTypeChatUpdatePermissions = 72,
+    MEGARequestTypeChatTruncate = 73,
+    MEGARequestTypeChatSetTitle = 74,
+    MEGARequestTypeSetMaxConnections = 75,
+    MEGARequestTypePauseTransfer = 76,
+    MEGARequestTypeMoveTransfer = 77,
+    MEGARequestTypeChatPresenceUrl = 78,
+    MEGARequestTypeRegisterPushNotification = 79,
+    MEGARequestTypeGetUserEmail = 80,
+    MEGARequestTypeAppVersion = 81,
+    MEGARequestTypeGetLocalSSLCertificate = 82,
+    MEGARequestTypeSendSignupLink = 83,
+    MEGARequestTypeQueryDns = 84,
+    MEGARequestTypeQueryGelb = 85, // Deprecated
+    MEGARequestTypeChatStats = 86,
+    MEGARequestTypeDownloadFile = 87,
+    MEGARequestTypeQueryTransferQuota = 88,
+    MEGARequestTypePasswordLink = 89,
+    MEGARequestTypeGetAchievements = 90,
+    MEGARequestTypeRestore = 91,
+    MEGARequestTypeRemoveVersions = 92,
+    MEGARequestTypeChatArchive = 93,
+    MEGARequestTypeWhyAmIBlocked = 94,
+    MEGARequestTypeContactLinkCreate = 95,
+    MEGARequestTypeContactLinkQuery = 96,
+    MEGARequestTypeContactLinkDelete = 97,
+    MEGARequestTypeFolderInfo = 98,
+    MEGARequestTypeRichLink = 99,
+    MEGARequestTypeKeepMeAlive = 100,
+    MEGARequestTypeMultiFactorAuthCheck = 101,
+    MEGARequestTypeMultiFactorAuthGet = 102,
+    MEGARequestTypeMultiFactorAuthSet = 103,
+    MEGARequestTypeAddBackup = 104,
+    MEGARequestTypeRemoveBackup = 105,
+    MEGARequestTypeTimer = 106,
+    MEGARequestTypeAbortCurrentBackup = 107,
+    MEGARequestTypeGetPSA = 108,
+    MEGARequestTypeFetchTimeZone = 109,
+    MEGARequestTypeUseralertAcknowledge = 110,
+    MEGARequestTypeChatLinkHandle = 111,
+    MEGARequestTypeChatLinkUrl = 112,
+    MEGARequestTypeSetPrivateMode = 113,
+    MEGARequestTypeAutojoinPublicChat = 114,
+    MEGARequestTypeCatchup = 115,
+    MEGARequestTypePublicLinkInformation = 116,
+    MEGARequestTypeGetBackgroundUploadURL = 117,
+    MEGARequestTypeCompleteBackgroundUpload = 118,
+    MEGARequestTypeCloudStorageUsed = 119,
+    MEGARequestTypeSendSMSVerificationCode = 120,
+    MEGARequestTypeCheckSMSVerificationCode = 121,
+    MEGARequestTypeGetRegisteredContacts = 122, // Deprecated
+    MEGARequestTypeGetCountryCallingCodes = 123,
+    MEGARequestTypeVerifyCredentials = 124,
+    MEGARequestTypeGetMiscFlags = 125,
+    MEGARequestTypeResendVerificationEmail = 126,
+    MEGARequestTypeSupportTicket = 127,
+    MEGARequestTypeRetentionTime = 128,
+    MEGARequestTypeResetSmsVerifiedNumber = 129,
+    MEGARequestTypeSendDevCommand = 130,
+    MEGARequestTypeGetBanners = 131,
+    MEGARequestTypeDismissBanner = 132,
+    MEGARequestTypeBackupPut = 133,
+    MEGARequestTypeBackupRemove = 134,
+    MEGARequestTypeBackupPutHeartbeat = 135,
+    MEGARequestTypeFetchAds = 136,
+    MEGARequestTypeQueryAds = 137,
+    MEGARequestTypeGetAttrNode = 138,
+    MEGARequestTypeLoadExternalDriveBackups = 139,
+    MEGARequestTypeCloseExternalDriveBackups = 140,
+    MEGARequestTypeGetDownloadUrls = 141,
+    MEGARequestTypeStartChatCall = 142,
+    MEGARequestTypeJoinChatCall = 143,
+    MEGARequestTypeEndChatCall = 144,
+    MEGARequestTypeGetFAUploadUrl = 145,
+    MEGARequestTypeExecuteOnThread = 146,
+    MEGARequestTypeGetChatOptions = 147,
+    MEGARequestTypeGetRecentActions = 148,
+    MEGARequestTypeCheckRecoveryKey = 149,
+    MEGARequestTypeSetMyBackups = 150,
+    MEGARequestTypePutSet = 151,
+    MEGARequestTypeRemoveSet = 152,
+    MEGARequestTypeFetchSet = 153,
+    MEGARequestTypePutSetElement = 154,
+    MEGARequestTypeRemoveSetElement = 155,
+    MEGARequestTypeRemoveOldBackupNodes = 156,
+    MEGARequestTypeSetSyncRunstate = 157,
+    MEGARequestTypeAddUpdateScheduledMeeting = 158,
+    MEGARequestTypeDelScheduledMeeting = 159,
+    MEGARequestTypeFetchScheduledMeeting = 160,
+    MEGARequestTypeFetchScheduledMeetingOccurrences = 161,
+    MEGARequestTypeOpenShareDialog = 162,
+    MEGARequestTypeUpgradeSecurity = 163,
+    MEGARequestTypePutSetElements = 164,
+    MEGARequestTypeRemoveSetElements = 165,
+    MEGARequestTypeExportSet = 166,
+    MEGARequestTypeExportedSetElement = 167,
+    MEGARequestTypeGetRecommenedProPlan = 168,
+    MEGARequestTypeBackupInfo = 169,
+    MEGARequestTypeBackupRemoveMD = 170,
+    MEGARequestTypeABTestActive = 171,
+    MEGARequestTypeGetVPNRegions = 172,
+    MEGARequestTypeGetVPNCredentials = 173,
+    MEGARequestTypePutVPNCredentials = 174,
+    MEGARequestTypeDeleteVPNCredentials = 175,
+    MEGARequestTypeCheckVPNCredentials = 176,
+    MEGARequestTypeGetSyncStallList = 177,
+    MEGARequestTypeFetchCreditCardInfo = 178,
+    MEGARequestTypeMoveToDebris = 179,
+    MEGARequestTypeRingIndividualInCall = 180,
+    MEGARequestTypeCreateNodeTree = 181,
+    MEGARequestTypeCreatePasswordManagerBase = 182,
+    MEGARequestTypeCreatePasswordNode = 183,
+    MEGARequestTypeUpdatePasswordNode = 184,
+    MEGARequestTypeGetNotifications = 185,
+    MEGARequestTypeTagNode = 186,
+    MEGARequestTypeAddMount = 187,
+    MEGARequestTypeDisableMount = 188,
+    MEGARequestTypeEnableMount = 189,
+    MEGARequestTypeRemoveMount = 190,
+    MEGARequestTypeSetMountFlags = 191,
+    MEGARequestTypeDelAttrUser = 192,
+    MEGARequestTypeBackupPauseMD = 194,
+    MEGARequestTypeBackupResumeMD = 195,
     MEGARequestTypeImportPasswordsFromFile = 196,
-    TotalOfRequestTypes = 202
+    MEGARequestTypeGetActiveSurveyTriggerActions = 197,
+    MEGARequestTypeGetSurvey = 198,
+    MEGARequestTypeAnswerSurvey = 199,
+    MEGARequestTypeChangeSyncRoot = 200,
+    MEGARequestTypeGetMyIP = 201,
+    MEGARequestTypeSetSyncUploadThrottleValues = 202,
+    MEGARequestTypeGetSyncUploadThrottleValues = 203,
+    MEGARequestTypeGetSyncUploadThrottleLimits = 204,
+    MEGARequestTypeCheckSyncUploadThrottledValues = 205,
+    MEGARequestTypeRunNetworkConnectivityTest = 206,
+    TotalOfRequestTypes = 207
 };
 
 typedef NS_ENUM (NSInteger, MEGANodeAccessLevel) {
@@ -677,6 +704,13 @@ typedef NS_ENUM (NSInteger, MEGANodeAccessLevel) {
  */
 @property (readonly, nonatomic, nullable) MEGAVPNCredentials *megaVpnCredentials;
 
+/**
+ * @brief Container class to store the results of a network connectivity test
+ *
+ * @return an object of MEGANetworkConnectivityTestResult
+ */
+@property (readonly, nonatomic, nullable) MEGANetworkConnectivityTestResults *megaNetworkConnectivityTestResults;
+
 /**
  * @brief Provide all available VPN Regions, including their details.
  *
