@@ -22,6 +22,8 @@
 
 #import "MEGAUser.h"
 
+NS_ASSUME_NONNULL_BEGIN
+
 /**
  * @brief List of MEGAUser objects.
  *
@@ -35,20 +37,7 @@
 /**
  * @brief The number of MEGAUser objects in the list
  */
-@property (readonly, nonatomic) NSNumber *size;
-
-/**
- * @brief Creates a copy of this MEGAUserList object.
- *
- * The resulting object is fully independent of the source MEGAUserList,
- * it contains a copy of all internal attributes, so it will be valid after
- * the original object is deleted.
- *
- * You are the owner of the returned object.
- *
- * @return Copy of the MEGAUserList object.
- */
-- (instancetype)clone;
+@property (readonly, nonatomic) NSInteger size;
 
 /**
  * @brief The MEGAUser at the position index in the MEGAUserList.
@@ -58,6 +47,7 @@
  * @param index Position of the MEGAUser that we want to get for the list.
  * @return MEGAUser at the position index in the list.
  */
-- (MEGAUser *)userAtIndex:(NSInteger)index;
+- (nullable MEGAUser *)userAtIndex:(NSInteger)index;
 
+NS_ASSUME_NONNULL_END
 @end
