@@ -35,7 +35,7 @@ public:
     void onUserAlertsUpdate(MegaApi* api, MegaUserAlertList *alerts) override;
     void onNodesUpdate(MegaApi* api, MegaNodeList *nodes) override;
     void onAccountUpdate(MegaApi* api) override;
-	void onReloadNeeded(MegaApi* api) override;
+    void onReloadNeeded(MegaApi* api);
     void onEvent(MegaApi* api, MegaEvent *e) override;
 
 #ifdef ENABLE_SYNC
