@@ -92,14 +92,6 @@ CURL_SOURCE_FOLDER=curl-${CURL_VERSION}
 CURL_DOWNLOAD_URL=http://curl.haxx.se/download/${CURL_SOURCE_FILE}
 CURL_SHA1="5ff2ecaa4a68ecc06434644ce76d9837e99e7d1d"
 
-C_ARES_VERSION=1.19.1
-C_ARES_VERSION2=1_19_1
-ARES_SOURCE_FILE=c-ares-${C_ARES_VERSION}.tar.gz
-ARES_SOURCE_FOLDER=c-ares-${C_ARES_VERSION}
-ARES_CONFIGURED=${CURL}/${ARES_SOURCE_FOLDER}/Makefile.inc
-ARES_DOWNLOAD_URL=https://github.com/c-ares/c-ares/releases/download/cares-${C_ARES_VERSION2}/${ARES_SOURCE_FILE}
-ARES_SHA1="99566278e4ed4b261891aa62c8b88227bf1a2823"
-
 CRASHLYTICS=crashlytics
 CRASHLYTICS_DOWNLOAD_URL=https://raw.githubusercontent.com/firebase/firebase-android-sdk/master/firebase-crashlytics-ndk/src/main/jni/libcrashlytics/include/crashlytics/external/crashlytics.h
 CRASHLYTICS_DOWNLOAD_URL_C=https://raw.githubusercontent.com/firebase/firebase-android-sdk/8f02834e94f8b24a7cf0f777562cad73c6b9a40f/firebase-crashlytics-ndk/src/main/jni/libcrashlytics/include/crashlytics/external/crashlytics.h
@@ -547,23 +539,14 @@ if [ ! -f ${OPENSSL}/${OPENSSL_SOURCE_FILE}.ready ]; then
 fi
 echo "* OpenSSL is ready"
 
-echo "* Setting up cURL with c-ares"
+echo "* Setting up cURL"
 if [ ! -f ${CURL}/${CURL_SOURCE_FILE}.ready ]; then
     echo "* Setting up cURL"
     downloadCheckAndUnpack ${CURL_DOWNLOAD_URL} ${CURL}/${CURL_SOURCE_FILE} ${CURL_SHA1} ${CURL}
     ln -sf ${CURL_SOURCE_FOLDER} ${CURL}/${CURL}
 
-    echo "* Setting up c-ares"
-    downloadCheckAndUnpack ${ARES_DOWNLOAD_URL} ${CURL}/${ARES_SOURCE_FILE} ${ARES_SHA1} ${CURL}
-    ln -sf ${ARES_SOURCE_FOLDER} ${CURL}/ares
-
-    # echo "* Patching c-ares to include crashlytics"
-    # if ! patch -R -p0 -s -f --dry-run ${CURL}/ares/src/lib/ares_android.c < ${CURL}/ares_android_c.patch &>> ${LOG_FILE}; then
-    #     patch -p0 ${CURL}/ares/src/lib/ares_android.c < ${CURL}/ares_android_c.patch &>> ${LOG_FILE}
-    # fi
-
     for ABI in ${BUILD_ARCHS}; do
-        echo "* Prebuilding cURL with c-ares for ${ABI}"
+        echo "* Prebuilding cURL for ${ABI}"
 
         setupEnv "${ABI}"
 
@@ -577,17 +560,9 @@ if [ ! -f ${CURL}/${CURL_SOURCE_FILE}.ready ]; then
 	        SSL_SUFFIX="openssl-android-x86"
 	    fi
 
-        pushd ${CURL}/ares &>> ${LOG_FILE}
-        LDFLAGS+="-Wl,-z,max-page-size=16384" ./configure --host "${TARGET_HOST}" --with-pic --disable-shared --prefix="${BASE_PATH}/${CURL}"/ares/ares-android-${ABI} &>> ${LOG_FILE}
-        make clean &>> ${LOG_FILE}
-        make -j${JOBS} &>> ${LOG_FILE}
-        make install &>> ${LOG_FILE}
-        popd &>> ${LOG_FILE}
-
         pushd ${CURL}/${CURL} &>> ${LOG_FILE}
 
-        LDFLAGS+="-Wl,-z,max-page-size=16384" LIBS=-lc++ ./configure --host "${TARGET_HOST}" --with-pic --disable-shared --prefix="${BASE_PATH}/${CURL}/${CURL}"/curl-android-${ABI} --with-ssl="${BASE_PATH}/${OPENSSL}/${OPENSSL}/${SSL_SUFFIX}" \
-        --enable-ares="${BASE_PATH}/${CURL}"/ares/ares-android-${ABI} ${CURL_EXTRA} &>> ${LOG_FILE}
+        LDFLAGS+="-Wl,-z,max-page-size=16384" LIBS=-lc++ ./configure --host "${TARGET_HOST}" --with-pic --disable-shared --prefix="${BASE_PATH}/${CURL}/${CURL}"/curl-android-${ABI} --with-ssl="${BASE_PATH}/${OPENSSL}/${OPENSSL}/${SSL_SUFFIX}" &>> ${LOG_FILE}
         make clean &>> ${LOG_FILE}
         make -j${JOBS} &>> ${LOG_FILE}
         make install &>> ${LOG_FILE}
@@ -598,7 +573,7 @@ if [ ! -f ${CURL}/${CURL_SOURCE_FILE}.ready ]; then
 
     touch ${CURL}/${CURL_SOURCE_FILE}.ready
 fi
-echo "* cURL with c-ares is ready"
+echo "* cURL is ready"
 
 echo "* Setting up ICU"
 if [ ! -f ${ICU}/${ICU_SOURCE_FILE}.ready ]; then
