@@ -3,6 +3,7 @@
 #include <string>
 
 #include <mega/fuse/common/mount_flags.h>
+#include <mega/fuse/common/query_forward.h>
 #include <mega/fuse/common/scoped_query_forward.h>
 
 namespace mega
@@ -16,8 +17,10 @@ struct MountFlags
 
     bool operator!=(const MountFlags& rhs) const;
 
+    static MountFlags deserialize(Query& query);
     static MountFlags deserialize(ScopedQuery& query);
 
+    void serialize(Query& query) const;
     void serialize(ScopedQuery& query) const;
 
     std::string mName;
