@@ -34,7 +34,7 @@ private: // mega::GfxProc implementations
     const char* supportedformats() override;
     const char* supportedvideoformats() override;
     bool readbitmap(const mega::LocalPath&, int) override;
-    bool resizebitmap(int, int, mega::string*) override;
+    bool resizebitmap(int, int, std::string*, Hint hint) override;
     void freebitmap() override;
 public:
     GfxProviderCG();
