@@ -49,9 +49,9 @@
 namespace mega {
 
 #ifdef _WIN32
-    const char* mega_inet_ntop(int af, const void* src, char* dst, int cnt);
+const char* mega_inet_ntop(int af, const void* src, char* dst, int cnt);
 #else
-    #define mega_inet_ntop inet_ntop
+#define mega_inet_ntop inet_ntop
 #endif
 
 // SSL public key pinning - active key
@@ -174,13 +174,6 @@ namespace mega {
 #define SFUSTATSSSLEXPONENTSIZE "\x03"
 #define SFUSTATSSSLEXPONENT "\x01\x00\x01"
 
-
-#define DNS_SERVERS "2001:4860:4860::8888,8.8.8.8," \
-                    "2001:4860:4860::8844,8.8.4.4," \
-                    "2606:4700:4700::1111,1.1.1.1," \
-                    "2606:4700:4700::1001,1.0.0.1," \
-                    "2620:fe::fe,9.9.9.9"
-
 class MEGA_API SpeedController
 {
 public:
@@ -311,9 +304,6 @@ struct MEGA_API HttpIO : public EventTrigger
     // get proxy settings from the system
     virtual Proxy *getautoproxy();
 
-    // get alternative DNS servers
-    void getMEGADNSservers(string* dnsservers, bool getfromnetwork);
-
     // set max download speed
     virtual bool setmaxdownloadspeed(m_off_t bpslimit);
 
@@ -330,6 +320,9 @@ struct MEGA_API HttpIO : public EventTrigger
 
     HttpIO();
     virtual ~HttpIO() { }
+
+    virtual void setproxy(const Proxy&);
+    virtual std::optional<Proxy> getproxy() const;
 };
 
 // outgoing HTTP request
@@ -477,7 +470,7 @@ struct MEGA_API HttpReq
 
     void setLogName(const std::string& newLogName)
     {
-        logname = newLogName + logname;
+        logname += newLogName;
     }
 
 private:
