@@ -364,6 +364,11 @@ class SimpleLogger
         copyToBuffer(value.begin(), static_cast<DiffType>(value.size()));
     }
 
+    void logValue(const std::string_view value)
+    {
+        copyToBuffer(value.begin(), static_cast<DiffType>(value.size()));
+    }
+
     void logValue(const ::mega::Error& value)   // ::mega:: when building MEGAchat on windows, else ambiguity errors
     {
         logValue(error(value));
@@ -480,6 +485,9 @@ public:
 #endif
     }
 
+    SimpleLogger(const SimpleLogger&) = delete;
+    SimpleLogger& operator=(const SimpleLogger&) = delete;
+
     static const char *toStr(LogLevel ll)
     {
         switch (ll)
@@ -838,4 +846,48 @@ public:
 extern ExternalLogger g_externalLogger;
 extern ExclusiveLogger g_exclusiveLogger;
 
+inline error logAndReturnError(const error e, const std::string_view msg)
+{
+    LOG_err << msg;
+    return e;
+}
+
+/**
+ * @brief A generic version of `logAndReturnError` where you can specify how the return value is
+ * instantiated from the error code
+ *
+ * @param returnGenerator A function that can be invoked with `const error` and generates the
+ * returned value
+ * @return The result from invoking returnGenerator(e)
+ */
+template<typename F>
+auto logAndReturnErrorGeneric(const error e, const std::string_view msg, F&& returnGenerator)
+    -> std::invoke_result_t<F, error>
+{
+    LOG_err << msg;
+    return returnGenerator(e);
+}
+
+/**
+ * @brief A helper factory function to generate callables with the same input parameters as
+ * `logAndReturnError` but with a custom return type, generated by resGenerator.
+ *
+ * @example
+ * const auto logAndError = generateLogAndReturnError(
+ *     [](const error e) -> std::pair<error, std::pair<std::string, unsigned>>
+ *     {
+ *         return {e, {"", 0}};
+ *     });
+ * return logAndError(API_EARGS, "Invalid handle"); // will return the pair
+ */
+template<typename F>
+constexpr auto generateLogAndReturnError(F&& resGenerator)
+    -> std::function<std::invoke_result_t<std::decay_t<F>, error>(error, std::string_view)>
+{
+    return [gen = std::forward<F>(resGenerator)](const error e, const std::string_view msg)
+    {
+        return logAndReturnErrorGeneric(e, msg, gen);
+    };
+}
+
 } // namespace
