@@ -300,9 +300,30 @@ public:
 
     sharedNode_vector searchNodes(const NodeSearchFilter& filter, int order, CancelToken cancelFlag, const NodeSearchPage& page);
 
-    std::set<std::string> getAllNodeTags(const char* searchString, CancelToken cancelFlag);
+    /*
+     * @brief
+     * Get all node tags below a specified node.
+     *
+     * @param cancelToken
+     * A token that can be used to terminate the query's execution prematurely.
+     *
+     * @param handles
+     * A set of handles specifying which nodes we want to list tags below.
+     *
+     * If undefined, the query will list tags below all root nodes.
+     *
+     * @param pattern
+     * An optional pattern that can be used to filter which tags we list.
+     *
+     * @returns
+     * std::nullopt on failure.
+     * std::set<std::string> on success.
+     */
+    auto getNodeTagsBelow(CancelToken cancelToken,
+                          const std::set<NodeHandle>& handles,
+                          const std::string& pattern = {}) -> std::optional<std::set<std::string>>;
 
-    sharedNode_vector getNodesByFingerprint(FileFingerprint& fingerprint);
+    sharedNode_vector getNodesByFingerprint(const FileFingerprint& fingerprint);
     sharedNode_vector getNodesByOrigFingerprint(const std::string& fingerprint, Node *parent);
     std::shared_ptr<Node> getNodeByFingerprint(FileFingerprint &fingerprint);
 
@@ -547,8 +568,6 @@ private:
     sharedNode_vector getChildren_internal(const NodeSearchFilter& filter, int order, CancelToken cancelFlag, const NodeSearchPage& page);
     sharedNode_vector getRecentNodes_internal(const NodeSearchPage& page, m_time_t since);
 
-    std::set<std::string> getAllNodeTags_internal(const char* searchString, CancelToken cancelFlag);
-
     // node temporary in memory, which will be removed upon write to DB
     std::shared_ptr<Node> mNodeToWriteInDb;
 
@@ -586,7 +605,7 @@ private:
     sharedNode_list getChildren_internal(const Node* parent,
                                          CancelToken cancelToken = CancelToken());
 
-    sharedNode_vector getNodesByFingerprint_internal(FileFingerprint& fingerprint);
+    sharedNode_vector getNodesByFingerprint_internal(const FileFingerprint& fingerprint);
     sharedNode_vector getNodesByOrigFingerprint_internal(const std::string& fingerprint, Node *parent);
     std::shared_ptr<Node> getNodeByFingerprint_internal(FileFingerprint &fingerprint);
     std::shared_ptr<Node> childNodeByNameType_internal(const Node *parent, const std::string& name, nodetype_t nodeType);
