@@ -32,6 +32,8 @@
 
 #ifdef ENABLE_SYNC
 #include "node.h"
+#include "syncinternals/syncinternals.h"
+#include "syncinternals/synciuploadthrottlingmanager.h"
 
 namespace mega {
 
@@ -118,7 +120,7 @@ public:
      * @brief In case this is an external backup, returns true if this is a backup sync and the
      * given path belongs to the external drive. Always true for non-external backups.
      */
-    bool isGoodPathForExternalBackup(const LocalPath& path)
+    bool isGoodPathForExternalBackup(const LocalPath& path) const
     {
         return !isExternal() || (isBackup() && mExternalDrivePath.isContainingPathOf(path));
     }
@@ -243,6 +245,13 @@ private:
     SyncRunState mKnownRunState = SyncRunState::Pending;
 };
 
+std::pair<error, SyncConfig> buildSyncConfig(const SyncConfig::Type syncType,
+                                             const std::string& localPath,
+                                             const std::string& name,
+                                             const std::string& drivePath,
+                                             const handle nodeHandle,
+                                             MegaClient& client);
+
 // Convenience.
 using SyncConfigVector = vector<SyncConfig>;
 struct Syncs;
@@ -481,13 +490,16 @@ public:
 
     // Remember which Nodes we created from upload,
     // until the corresponding LocalNodes are updated.
-    void addExpectedUpload(NodeHandle parentHandle, const string& name, weak_ptr<SyncUpload_inClient>);
-    void removeExpectedUpload(NodeHandle parentHandle, const string& name);
-    shared_ptr<SyncUpload_inClient> isNodeAnExpectedUpload(NodeHandle parentHandle, const string& name);
+    void addExpectedUpload(NodeHandle parentHandle,
+                           const string& name,
+                           weak_ptr<SyncUpload_inClient>);
+    virtual void removeExpectedUpload(NodeHandle parentHandle, const string& name);
+    shared_ptr<SyncUpload_inClient> isNodeAnExpectedUpload(NodeHandle parentHandle,
+                                                           const string& name);
 
-    void transferBegin(direction_t direction, m_off_t numBytes);
-    void transferComplete(direction_t direction, m_off_t numBytes);
-    void transferFailed(direction_t direction, m_off_t numBytes);
+    virtual void transferBegin(direction_t direction, m_off_t numBytes);
+    virtual void transferComplete(direction_t direction, m_off_t numBytes);
+    virtual void transferFailed(direction_t direction, m_off_t numBytes);
 
     // Return a snapshot of this sync's current transfer counts.
     SyncTransferCounts transferCounts() const;
@@ -500,9 +512,23 @@ public:
     LocalPath syncTmpFolder() const;
     void setSyncTmpFolder(const LocalPath&);
 
-    SyncThreadsafeState(handle backupId, MegaClient* client, bool canChangeVault) : mClient(client), mBackupId(backupId), mCanChangeVault(canChangeVault)  {}
-    handle backupId() const { return mBackupId; }
-    MegaClient* client() const { return mClient; }
+    SyncThreadsafeState(handle backupId, MegaClient* client, bool canChangeVault):
+        mClient(client),
+        mBackupId(backupId),
+        mCanChangeVault(canChangeVault)
+    {}
+
+    virtual ~SyncThreadsafeState(){};
+
+    handle backupId() const
+    {
+        return mBackupId;
+    }
+
+    MegaClient* client() const
+    {
+        return mClient;
+    }
 };
 
 class MEGA_API Sync
@@ -690,7 +716,18 @@ public:
     // How deep is this sync's cloud root?
     unsigned mCurrentRootDepth = 0;
 
-    Sync(UnifiedSync&, const string&, const LocalPath&, bool, const string& logname, SyncError& e);
+    /**
+     * @brief Sync constructor
+     *
+     * @param[in,out] us the `UnifiedSync` object in which the new Sync will be stored. It is used
+     * to read the configuration and instantiate the Sync accordingly. Also, non-const references to
+     * some of its members (e.g. `syncs`) are also saved in the object. Additionally, some members
+     * of the config (owned by the `UnifiedSync`) are also modified inside this constructor.
+     * @param[in] logname A prefix to include in the logging messages involving this object.
+     * @param[out] e An output error code. If the object is constructed successfully, NO_SYNC_ERROR
+     */
+    Sync(UnifiedSync& us, const std::string& logname, SyncError& e);
+
     ~Sync();
 
     /**
@@ -1370,8 +1407,43 @@ struct Syncs
 
     void transferPauseFlagsUpdated(bool downloadsPaused, bool uploadsPaused);
 
-    // returns a copy of the config, for thread safety
-    bool syncConfigByBackupId(handle backupId, SyncConfig&) const;
+private:
+    /**
+     * @brief Searches for a SyncConfig with the given backupID and executes an optional action
+     * callback.
+     *
+     * This method is expected to be called out of the sync thread.
+     *
+     * Acquires the lock on mSyncVec, searches for a SyncConfig with the given backupId,
+     * and if found, invokes the provided callable under lock.
+     *
+     * @tparam SyncConfigCallable Callable type that accepts a SyncConfig type param.
+     * @param backupId The backupID to search for.
+     * @param completion A callable that is invoked with the SyncConfig if a match is found.
+     *                   If set to nullptr, no callback is executed.
+     * @return true if a matching SyncConfig was found, false otherwise.
+     */
+    template<typename SyncConfigCallable>
+    bool ifFoundSyncConfigByBackupId(const handle backupId, SyncConfigCallable&& action) const;
+
+public:
+    /**
+     * @brief Checks whether a sync configuration with the specified backupID exists.
+     *
+     * @return true if a matching SyncConfig was found, false otherwise.
+     * @see findSyncConfigByBackupId()
+     */
+    bool hasSyncConfigByBackupId(const handle backupId) const;
+
+    /**
+     * @brief Retrieves a copy of the sync configuration if exists for the specified backupID.
+     *
+     * @param syncConfig Output parameter that receives the found configuration (copied for thread
+     * safety).
+     * @return true if a matching SyncConfig was found, false otherwise.
+     * @see findSyncConfigByBackupId()
+     */
+    bool syncConfigByBackupId(const handle backupId, SyncConfig& syncConfig) const;
 
     void purgeRunningSyncs();
     void loadSyncConfigsOnFetchnodesComplete(bool resetSyncConfigStore);
@@ -1503,13 +1575,69 @@ public:
     // maps local fsid to corresponding LocalNode* (s)
     fsid_localnode_map localnodeBySyncedFsid;
     fsid_localnode_map localnodeByScannedFsid;
-    LocalNode* findLocalNodeBySyncedFsid(const fsfp_t& fsfp, mega::handle fsid, const LocalPath& originalpath, nodetype_t type, const FileFingerprint& fp,
-                                         std::function<bool(LocalNode* ln)> extraCheck, handle owningUser, bool& foundExclusionUnknown);
-    LocalNode* findLocalNodeByScannedFsid(const fsfp_t& fsfp, mega::handle fsid, const LocalPath& originalpath, nodetype_t type, const FileFingerprint* fp,
-                                         std::function<bool(LocalNode* ln)> extraCheck, handle owningUser, bool& foundExclusionUnknown);
 
-    void setSyncedFsidReused(const fsfp_t& fsfp, mega::handle fsid);
-    void setScannedFsidReused(const fsfp_t& fsfp, mega::handle fsid);
+    /**
+     * @brief Finds a LocalNode by its synced FSID.
+     *
+     * Searches for a LocalNode in the map of synced FSIDs.
+     * This is used to detect moves, while avoiding mismatches caused by FSID reuse.
+     *
+     * @param fsid The FSID of the node to be searched on the localnode map.
+     * @param targetNodeAttributes The necessary node attributes of the node looking for another
+     * node matching by FSID.
+     * @param originalPathForLogging The original path being processed for context in logs.
+     * @param extraCheck An optional callable for additional filtering of LocalNodes.
+     * @param onFingerprintMismatchDuringPutnodes Optional operation for a LocalNode that
+     * has been excluded due to fingerprint mismatch, but the source node has a putnodes operation
+     * ongoing for an upload which matches fingerprint with the target node.
+     * The param is not const intentionally, in case it needs to be considered as a potential
+     * source node, taking into account that there is a fingerprint match for the ongoing upload.
+     *
+     * @return A pair with:
+     *         bool - indicating whether an unknown exclusion was encountered. This may occur during
+     * eg. the first pass of the tree after loading from Suspended state and the corresponding node
+     * is later in the tree. The caller should decide whether to pospone the logic if an unknown
+     * exclusion was found for some node.
+     *         LocalNode* - pointer to the matching LocalNode, or nullptr if no match is found.
+     *
+     * @see findLocalNodeByFsid()
+     */
+    std::pair<bool, LocalNode*> findLocalNodeBySyncedFsid(
+        const handle fsid,
+        const NodeMatchByFSIDAttributes& targetNodeAttributes,
+        const LocalPath& originalPathForLogging,
+        std::function<bool(const LocalNode&)> extraCheck = nullptr,
+        std::function<void(LocalNode*)> onFingerprintMismatchDuringPutnodes = nullptr) const;
+
+    /**
+     * @brief Finds a LocalNode by its scanned FSID.
+     *
+     * Searches for a LocalNode in the map of scanned FSIDs. This is used to detect
+     * moves and avoid mismatches caused by FSID reuse.
+     *
+     * @param fsid The FSID of the node to be searched on the localnode map.
+     * @param targetNodeAttributes The necessary node attributes of the node looking for another
+     * node matching by FSID.
+     * @param originalPathForLogging The original path being processed for context in logs.
+     * @param extraCheck An optional callable for additional filtering of LocalNodes.
+     *
+     * @return A pair with:
+     *         bool - indicating whether an unknown exclusion was encountered. This may occur during
+     * eg. the first pass of the tree after loading from Suspended state and the corresponding node
+     * is later in the tree. The caller should decide whether to pospone the logic if an unknown
+     * exclusion was found for some node.
+     *         LocalNode* - pointer to the matching LocalNode, or nullptr if no match is found.
+     *
+     * @see findLocalNodeByFsid()
+     */
+    std::pair<bool, LocalNode*> findLocalNodeByScannedFsid(
+        const handle fsid,
+        const NodeMatchByFSIDAttributes& targetNodeAttributes,
+        const LocalPath& originalPathForLogging,
+        std::function<bool(const LocalNode&)> extraCheck = nullptr) const;
+
+    void setSyncedFsidReused(const fsfp_t& fsfp, const handle fsid);
+    void setScannedFsidReused(const fsfp_t& fsfp, const handle fsid);
 
     // maps nodehandle to corresponding LocalNode* (s)
     nodehandle_localnode_map localnodeByNodeHandle;
@@ -1690,8 +1818,6 @@ public:
     std::chrono::steady_clock::time_point lastSyncStallsCount{std::chrono::steady_clock::now()};
     static const std::chrono::milliseconds MIN_DELAY_BETWEEN_SYNC_STALLS_OR_CONFLICTS_COUNT;
     static const std::chrono::milliseconds MAX_DELAY_BETWEEN_SYNC_STALLS_OR_CONFLICTS_COUNT;
-    static const std::chrono::milliseconds MIN_DELAY_BETWEEN_SYNC_VERBOSE_TIMED; // 5 secs
-    static const std::chrono::milliseconds TIME_WINDOW_FOR_SYNC_VERBOSE_TIMED; // 1 sec
 
     // Lock-free count of syncs currently active.
     std::atomic<unsigned> mNumSyncsActive{0u};
@@ -1732,10 +1858,23 @@ private:
     bool checkSyncsScanningWasComplete_inThread(); // Iterate through syncs, calling Sync::checkScanningWasComplete(). Returns false if any sync returns false.
     void unsetSyncsScanningWasComplete_inThread(); // Unset scanningWasComplete flag for every sync.
 
-    // actually start the sync (on sync thread)
-    void startSync_inThread(UnifiedSync& us, const string& debris, const LocalPath& localdebris,
-        bool inshare, bool isNetwork, const LocalPath& rootpath,
-        std::function<void(error, SyncError, handle)> completion, const string& logname);
+    /**
+     * @brief Instantiates the Sync object and stores it in the given unified sync
+     *
+     * @param[in,out] us The UnifiedSync with the configuration needed to create the sync (mConfig).
+     * It will be modified to store the new sync and to modify its state depending on the results.
+     * @param completion A function to be called once the process finishes with or without errors.
+     * The parameter passed to the callable are:
+     * - `error`: An error code indicating the result of the initiation.
+     * - `SyncError`: A detailed sync error code providing more context.
+     * - `handle`: the backup id of the sync being initiated
+     * @param logname The name to be passed to the Sync constructor. That will be stored in the
+     * `Sync::syncname` member and used as prefix in the logged messages.
+     */
+    void startSync_inThread(UnifiedSync& us,
+                            std::function<void(error, SyncError, handle)> completion,
+                            const string& logname);
+
     void prepareForLogout_inThread(bool keepSyncsConfigFile, std::function<void()> clientCompletion);
     void locallogout_inThread(bool removecaches, bool keepSyncsConfigFile, bool reopenStoreAfter);
     void loadSyncConfigsOnFetchnodesComplete_inThread(bool resetSyncConfigStore);
@@ -1779,6 +1918,14 @@ private:
             handle* owningUser = nullptr,
             vector<pair<handle, int>>* sdsBackups = nullptr);
 
+    /**
+     * @brief Check if the given cloud node is an in-share
+     *
+     * @param cn The cloud node to check
+     * @return true if the node is found and it is an inshare
+     */
+    bool isCloudNodeInShare(const CloudNode& cn);
+
     bool lookupCloudChildren(NodeHandle h, vector<CloudNode>& cloudChildren);
 
     // Query whether a specified child is definitely excluded.
@@ -1986,6 +2133,18 @@ private:
     std::atomic<bool> mUploadsPaused {false};
     std::atomic<bool> mTransferPauseFlagsChanged {false};
 
+    // Local sds values for full-syncs
+    using SyncsDesiredStates = std::vector<std::pair<handle, int>>;
+    SyncsDesiredStates mSdsBackupsFullSync;
+    mutable std::mutex mSdsBackupsFullSyncMutex;
+
+    /**
+     * @brief Shared pointer to IUploadThrottlingManager, in charge of everything related to upload
+     * throttling.
+     * @see IUploadThrottlingManager.
+     */
+    std::shared_ptr<IUploadThrottlingManager> mThrottlingManager;
+
     // Responsible for tracking when to send sync/backup heartbeats
     unique_ptr<BackupMonitor> mHeartBeatMonitor;
 
@@ -2087,6 +2246,155 @@ public:
         // Let the caller know if any syncs match our predicate.
         return notifier.get_future().get();
     }
+
+    // Syncs Upload Throttling
+
+    /**
+     * @brief Processes the delayed uploads.
+     * Expected to be called on the sync thread.
+     *
+     * When a delayed upload is processed after the required throttling time, it is enqueued as a
+     * regular upload to be processed in the client.
+     *
+     * @see IUploadThrottlingManager::processDelayedUploads()
+     * @see queueClient()
+     */
+    void processDelayedUploads();
+
+    /**
+     * @see IUploadThrottlingManager::addToDelayedUploads()
+     * Expected to be called on the sync thread.
+     */
+    void addToDelayedUploads(DelayedSyncUpload&& delayedUpload);
+
+    /**
+     * @see IUploadThrottlingManager::uploadCounterInactivityExpirationTime()
+     * Expected to be called on the sync thread.
+     */
+    std::chrono::seconds uploadCounterInactivityExpirationTime() const;
+
+    /**
+     * @see IUploadThrottlingManager::throttleUpdateRate()
+     * Expected to be called on the sync thread.
+     */
+    std::chrono::seconds throttleUpdateRate() const;
+
+    /**
+     * @see IUploadThrottlingManager::maxUploadsBeforeThrottle()
+     * Expected to be called on the sync thread.
+     */
+    unsigned maxUploadsBeforeThrottle() const;
+
+    /**
+     * @brief Sets the throttleUpdateRate configurable value for upload throttling.
+     *
+     * Method to be executed out of the sync thread. The logic is enqueued to be later called within
+     * the sync thread.
+     *
+     * @param completion The completion function to be called after the operations finishes.
+     * Error values:
+     * - API_OK: Value was updated correctly.
+     * - API_EARGS: Value was below or above throttleUpdateRate lower/upper limits.
+     */
+    void setThrottleUpdateRate(std::chrono::seconds throttleUpdateRate,
+                               std::function<void(const error)>&& completion);
+
+    /**
+     * @brief Sets the maxUploadsBeforeThrottle configurable value for upload throttling.
+     *
+     * Method to be executed out of the sync thread. The logic is enqueued to be later called within
+     * the sync thread.
+     *
+     * @param completion The completion function to be called after the operations finishes.
+     * Error values:
+     * - API_OK: Value was updated correctly.
+     * - API_EARGS: Value was below or above maxUploadsBeforeThrottle lower/upper limits.
+     */
+    void setMaxUploadsBeforeThrottle(const unsigned maxUploadsBeforeThrottle,
+                                     std::function<void(const error)>&& completion);
+
+    /**
+     * @brief Retrieves the upload throttling configurable values: throttleUpdateRate and
+     * maxUploadsBeforeThrottle.
+     *
+     * Method to be executed out of the sync thread. The logic is enqueued to be later called within
+     * the sync thread.
+     *
+     * @param completion The completion function to be called after the operations finishes.
+     * @return a pair with throttleUpdateRate, maxUploadsBeforeThrottle.
+     */
+    void uploadThrottleValues(
+        std::function<void(const std::chrono::seconds /* throttleUpdateRate */,
+                           const unsigned /* maxUploadsBeforeThrottle */)>&& completion);
+
+    /**
+     * @brief Retrieves the lower/upper limits for the upload throttling configurable values.
+     *
+     * Method to be executed out of the sync thread. The logic is enqueued to be later called within
+     * the sync thread.
+     *
+     * @param completion The completion function to be called after the operations finishes.
+     */
+    void uploadThrottleValuesLimits(std::function<void(ThrottleValueLimits&&)>&& completion);
+
+    /**
+     * @brief Checks whether or not there are delayed/throttled uploads waiting for processing.
+     *
+     * Method to be executed out of the sync thread. The logic is enqueued to be later called within
+     * the sync thread.
+     *
+     * @param completion The completion function to be called after the operations finishes.
+     */
+    void checkSyncUploadsThrottled(std::function<void(const bool)>&& completion);
+
+    /**
+     * @brief Sets the throttling manager object.
+     *
+     * Method to be executed out of the sync thread. The logic is enqueued to be later called within
+     * the sync thread.
+     *
+     * @param uploadThrottlingManager A valid shared_ptr to IUploadThrottlingManager.
+     * @param completion The completion function to be called after the operations finishes.
+     * Error values:
+     * - API_OK: New IUploadThrottlingManager was set correctly.
+     * - API_EARGS: uploadThrottlingManager is not a valid pointer.
+     */
+    void setThrottlingManager(std::shared_ptr<IUploadThrottlingManager> uploadThrottlingManager,
+                              std::function<void(const error)>&& completion);
+
+    /**
+     * @brief Sets the sds value for the syncs
+     *
+     * Method to be executed out of the sync thread. The value will be copied to be used later in
+     * the sync thread, protected by the mSdsBackupsFullSyncMutex mutex.
+     *
+     * @param sds New *!sds user attribute value. It can be null.
+     */
+    void setSdsBackupsFullSync(const std::unique_ptr<string_map>& sds);
+
+private:
+    /**
+     * @brief Checks the throttling manager validity.
+     *
+     * The throttling manager is always expected to be valid since the constructor.
+     * The method writes a log error and also performs an assert in case of the throttling manager
+     * being nullptr.
+     *
+     * This method is expected to be called on the sync thread. It also asserts that.
+     */
+    void assertThrottlingManagerIsValid() const;
+
+    /**
+     * @brief Gets the sds value for the full-sync with the sds node attribute format
+     *
+     * Internal method intended to be used in the sync thread. The returned values are set using
+     * setSdsBackupsFullSync method. Protected by the mSdsBackupsFullSyncMutex
+     *
+     * @see setSdsBackupsFullSync
+     *
+     * @return vector with the latest known *!sds user attribute values.
+     */
+    SyncsDesiredStates getSdsBackupsFullSync() const;
 };
 
 class OverlayIconCachedPaths
@@ -2148,7 +2456,6 @@ public:
     }
 };
 
-
 } // namespace
 
 #endif
