@@ -171,7 +171,11 @@ InodeDB::Queries::Queries(Database& database)
 
     mGetModifiedByID = "select modified from inodes where id = :id";
 
-    mGetModifiedInodes = "select * from inodes where modified = 1";
+    mGetModifiedInodes = "select * "
+                         "  from inodes "
+                         " where modified = 1 "
+                         "   and (handle is not null "
+                         "        or parent_handle is not null)";
 
     mGetNextInodeID = "select next from inode_id";
 
@@ -883,12 +887,16 @@ void InodeDB::handle(FileInode& file,
                      NodeHandle& oldHandle,
                      NodeHandle newHandle)
 {
-    // Acquire locks.
-    auto guard = lockAll(mContext.mDatabase, *this);
-
     // Sanity.
     assert(!newHandle.isUndef());
 
+    // Handle hasn't changed.
+    if (oldHandle == newHandle)
+        return;
+
+    // Acquire locks.
+    auto guard = lockAll(mContext.mDatabase, *this);
+
     // Swap new handle into place.
     std::swap(oldHandle, newHandle);
 
@@ -903,10 +911,6 @@ void InodeDB::handle(FileInode& file,
     query.param(":parent_handle") = nullptr;
     query.execute();
 
-    // Handle hasn't changed.
-    if (oldHandle == newHandle)
-        return transaction.commit();
-
     // Sanity.
     assert(!mByHandle.count(oldHandle));
 
@@ -1060,7 +1064,7 @@ ErrorOr<MakeInodeResult> InodeDB::makeDirectory(const platform::Mount&,
 
     // Couldn't create the directory.
     if (!result)
-        return result.error();
+        return unexpected(result.error());
 
     // Convenience.
     auto info = std::move(*result);
@@ -1099,7 +1103,7 @@ ErrorOr<MakeInodeResult> InodeDB::makeFile(const platform::Mount& mount,
 
     // Couldn't add a new file to the cache.
     if (!fileInfo)
-        return fileInfo.error();
+        return unexpected(fileInfo.error());
 
     // Create a description of our new file.
     NodeInfo info;
@@ -1969,6 +1973,11 @@ void InodeDB::EventObserver::added(const NodeEvent& event)
     // Node replaces an in-memory inode.
     if (auto ref = mInodeDB.child(name, parentHandle, MemoryOnly))
     {
+        // Node is a new version of this inode.
+        if (ref->handle() == handle)
+            return;
+
+        // Node replaces the inode.
         FUSEDebugF("%s (%s) [%s] replaces warm inode %s",
                    name.c_str(),
                    toNodeHandle(handle).c_str(),
