@@ -31,17 +31,17 @@ namespace fuse
 
 MountDB::Queries::Queries(Database& mDatabase)
   : mAddMount(mDatabase.query())
-  , mGetMountByPath(mDatabase.query())
-  , mGetMountFlagsByPath(mDatabase.query())
-  , mGetMountInodeByPath(mDatabase.query())
-  , mGetMountPathsByName(mDatabase.query())
-  , mGetMountStartupStateByPath(mDatabase.query())
+  , mGetMountByName(mDatabase.query())
+  , mGetMountFlagsByName(mDatabase.query())
+  , mGetMountInodeByName(mDatabase.query())
+  , mGetMountPathByName(mDatabase.query())
+  , mGetMountStartupStateByName(mDatabase.query())
   , mGetMounts(mDatabase.query())
   , mGetMountsEnabledAtStartup(mDatabase.query())
-  , mRemoveMountByPath(mDatabase.query())
+  , mRemoveMountByName(mDatabase.query())
   , mRemoveTransientMounts(mDatabase.query())
-  , mSetMountFlagsByPath(mDatabase.query())
-  , mSetMountStartupStateByPath(mDatabase.query())
+  , mSetMountFlagsByName(mDatabase.query())
+  , mSetMountStartupStateByName(mDatabase.query())
 {
     mAddMount = "insert into mounts values ( "
                 "  :enable_at_startup, "
@@ -52,54 +52,54 @@ MountDB::Queries::Queries(Database& mDatabase)
                 "  :read_only "
                 ")";
 
-    mGetMountByPath = "select * from mounts where path = :path";
+    mGetMountByName = "select * from mounts where name = :name";
 
-    mGetMountFlagsByPath = "select enable_at_startup "
+    mGetMountFlagsByName = "select enable_at_startup "
                            "     , name "
                            "     , persistent "
                            "     , read_only "
                            "  from mounts "
-                           " where path = :path";
+                           " where name = :name";
 
-    mGetMountInodeByPath = "select id from mounts where path = :path";
+    mGetMountInodeByName = "select id from mounts where name = :name";
 
-    mGetMountPathsByName = "select path from mounts where name = :name";
+    mGetMountPathByName = "select path from mounts where name = :name";
 
-    mGetMountStartupStateByPath = "select enable_at_startup "
+    mGetMountStartupStateByName = "select enable_at_startup "
                                   "     , persistent "
                                   "  from mounts "
-                                  " where path = :path";
+                                  " where name = :name";
 
     mGetMounts = "select * from mounts";
 
-    mGetMountsEnabledAtStartup = "select path "
+    mGetMountsEnabledAtStartup = "select name "
                                  "  from mounts "
                                  " where enable_at_startup = true "
                                  "   and persistent = true";
 
-    mRemoveMountByPath = "delete from mounts where path = :path";
+    mRemoveMountByName = "delete from mounts where name = :name";
 
     mRemoveTransientMounts = "delete from mounts where persistent = false";
 
-    mSetMountFlagsByPath = "update mounts "
+    mSetMountFlagsByName = "update mounts "
                            "   set enable_at_startup = :enable_at_startup "
                            "     , name = :name "
                            "     , persistent = :persistent "
                            "     , read_only = :read_only "
-                           " where path = :path";
+                           " where name = :current_name";
 
-    mSetMountStartupStateByPath =
+    mSetMountStartupStateByName =
       "update mounts "
       "   set enable_at_startup = :enable_at_startup "
       "     , persistent = :persistent "
-      " where path = :path";
+      " where name = :name";
 }
 
 MountResult MountDB::check(const MountInfo& info)
 {
     // Convenience.
     auto& handle = info.mHandle;
-    auto& name = info.mFlags.mName;
+    auto& name = info.name();
 
     // User's specified a bogus node handle.
     if (handle.isUndef())
@@ -163,7 +163,7 @@ try
     mContext.mFileCache.current();
 
     // What mounts should we try and enable?
-    std::vector<LocalPath> mounts;
+    std::vector<std::string> mounts;
 
     // Compute list of mounts to enable.
     {
@@ -175,21 +175,21 @@ try
         // What mounts should be enabled at startup?
         query.execute();
 
-        // Collect paths of enabled mounts.
+        // Collect names of enabled mounts.
         for ( ; query; ++query)
-            mounts.emplace_back(query.field("path"));
+            mounts.emplace_back(query.field("name"));
     }
 
     // Try and enable each mount.
-    for (auto& path : mounts)
+    for (auto& name : mounts)
     {
         MountEvent event;
 
-        event.mPath = path;
+        event.mName = name;
         event.mType = MOUNT_ENABLED;
 
         // Try and enable the mount.
-        event.mResult = enable(path, false);
+        event.mResult = enable(name, false);
 
         // Emit event.
         client().emitEvent(event);
@@ -198,7 +198,7 @@ try
         if (event.mResult != MOUNT_SUCCESS)
         {
             FUSEWarningF("Unable to enable persistent mount \"%s\" due to error: %s",
-                         path.toPath(false).c_str(),
+                         name.c_str(),
                          toString(event.mResult));
 
             // Try and enable the next mount.
@@ -207,7 +207,7 @@ try
 
         // Mount's been enabled.
         FUSEInfoF("Successfully enabled persistent mount \"%s\"",
-                  path.toPath(false).c_str());
+                  name.c_str());
     }
 }
 catch (std::runtime_error& exception)
@@ -379,15 +379,15 @@ try
     auto guard = lockAll(mContext.mDatabase, *this);
 
     auto transaction = mContext.mDatabase.transaction();
-    auto query = transaction.query(mQueries.mGetMountInodeByPath);
+    auto query = transaction.query(mQueries.mGetMountInodeByName);
 
-    // Make sure the path isn't already associated with a mount.
-    query.param(":path") = info.mPath;
+    // Make sure the name isn't already associated with a mount.
+    query.param(":name") = info.name();
     query.execute();
 
-    // A mount's already associated with this path.
+    // A mount's already associated with this name.
     if (query)
-        return MOUNT_EXISTS;
+        return MOUNT_NAME_TAKEN;
 
     // Add the description to the database.
     query = transaction.query(mQueries.mAddMount);
@@ -493,18 +493,18 @@ MountInfoPtr MountDB::contains(const LocalPath& path,
         *relativePath = path.subpathFrom(index);
 
     // Return description to caller.
-    return std::make_unique<MountInfo>(std::move(*mount));
+    return std::make_unique<MountInfo>(*mount);
 }
 
 void MountDB::disable(MountDisabledCallback callback,
-                      const LocalPath& path,
+                      const std::string& name,
                       bool remember)
 {
     auto guard = lockAll(mContext.mDatabase, *this);
 
     MountEvent event;
 
-    event.mPath = path;
+    event.mName = name;
     event.mType = MOUNT_DISABLED;
 
     // Emits the event and queues callback for execution.
@@ -531,17 +531,16 @@ void MountDB::disable(MountDisabledCallback callback,
     try
     {
         auto transaction = mContext.mDatabase.transaction();
-        auto query = transaction.query(mQueries.mGetMountStartupStateByPath);
+        auto query = transaction.query(mQueries.mGetMountStartupStateByName);
 
         // Query the mount's startup state.
-        query.param(":path") = path;
+        query.param(":name") = name;
         query.execute();
 
         // No mount associated with specified path.
         if (!query)
         {
-            FUSEErrorF("No mount associated with path: %s",
-                       path.toPath(false).c_str());
+            FUSEErrorF("No mount associated with name: %s", name.c_str());
 
             return emitEvent(MOUNT_UNKNOWN);
         }
@@ -554,10 +553,10 @@ void MountDB::disable(MountDisabledCallback callback,
         persistent = persistent || remember;
 
         // Update the mount's startup state.
-        query = transaction.query(mQueries.mSetMountStartupStateByPath);
+        query = transaction.query(mQueries.mSetMountStartupStateByName);
 
         query.param(":enable_at_startup") = enableAtStartup;
-        query.param(":path") = path;
+        query.param(":name") = name;
         query.param(":persistent") = persistent;
 
         query.execute();
@@ -565,7 +564,7 @@ void MountDB::disable(MountDisabledCallback callback,
         transaction.commit();
 
         // Is the mount currently enabled?
-        auto mount = this->mount(path);
+        auto mount = this->mount(name);
 
         // Mount's not enabled: We're done.
         if (!mount)
@@ -588,7 +587,7 @@ void MountDB::disable(MountDisabledCallback callback,
     {
         // Unexpected error while disabling mount.
         FUSEErrorF("Unable to disable mount %s: %s",
-                   path.toPath(false).c_str(),
+                   name.c_str(),
                    exception.what());
 
         emitEvent(MOUNT_UNEXPECTED);
@@ -669,27 +668,26 @@ void MountDB::each(std::function<void(platform::Mount&)> function)
         function(*mount);
 }
 
-MountResult MountDB::enable(const LocalPath& path, bool remember)
+MountResult MountDB::enable(const std::string& name, bool remember)
 try
 {
     auto lock = lockAll(mContext.mDatabase, *this);
 
     // The mount associated with this path is already enabled.
-    if (mount(path))
+    if (mount(name))
         return MOUNT_SUCCESS;
 
     auto transaction = mContext.mDatabase.transaction();
-    auto query = transaction.query(mQueries.mGetMountByPath);
+    auto query = transaction.query(mQueries.mGetMountByName);
 
     // Check if the mount's present in the database.
-    query.param(":path") = path;
+    query.param(":name") = name;
     query.execute();
 
     // Mount's not in the database.
     if (!query)
     {
-        FUSEErrorF("No mount associated with path: %s",
-                   path.toPath(false).c_str());
+        FUSEErrorF("No mount associated with name: %s", name.c_str());
 
         return MOUNT_UNKNOWN;
     }
@@ -704,10 +702,10 @@ try
     flags.mPersistent |= remember;
 
     // Update the mount's startup state.
-    query = transaction.query(mQueries.mSetMountStartupStateByPath);
+    query = transaction.query(mQueries.mSetMountStartupStateByName);
 
     query.param(":enable_at_startup") = flags.mEnableAtStartup;
-    query.param(":path") = path;
+    query.param(":name") = name;
     query.param(":persistent") = flags.mPersistent;
 
     query.execute();
@@ -724,25 +722,41 @@ try
     auto result = check(info);
 
     // Description's no longer sane.
-    if (result != MOUNT_SUCCESS)
+    //
+    // The reason we don't bail on LOCAL_EXISTS is that this
+    // error will always be signaled when you try to enable
+    // multiple mounts that use the same path. It'd also be
+    // generated in some cases if multiple threads tried to
+    // enable the same mount at the same time.
+    //
+    // So, instead of bailing here, bail later if needed. That
+    // way, we can check if the mount's already been enabled and
+    // return success. Or, if two distinct mounts with the same
+    // path are being enabled, we can return a more meaningful
+    // LOCAL_TAKEN error below.
+    if (result != MOUNT_SUCCESS && result != MOUNT_LOCAL_EXISTS)
         return result;
 
     // Reacquire the lock.
     lock.lock();
 
     // Another thread's enabled the mount.
-    if (auto mount = this->mount(info.mPath))
+    if (auto mount = this->mount(name))
         return MOUNT_SUCCESS;
 
-    // Another thread's enabled a mount with the same name.
-    if (auto mount = this->mount(info.mFlags.mName))
+    // Another thread's enabled a mount with the same path.
+    if (auto mount = this->mount(info.mPath))
     {
-        FUSEErrorF("Name %s already taken by mount: %s",
-                   info.mFlags.mName.c_str(),
-                   mount->path().toPath(false).c_str());
+        FUSEErrorF("Path %s already taken by mount: %s",
+                   info.mPath.toPath(false).c_str(),
+                   mount->name().c_str());
 
-        return MOUNT_NAME_TAKEN;
+        return MOUNT_LOCAL_TAKEN;
     }
+    
+    // Mount path might already exist on disk.
+    if (result != MOUNT_SUCCESS)
+        return result;
 
     // Convenience.
     auto& self = static_cast<platform::MountDB&>(*this);
@@ -752,8 +766,8 @@ try
 
     // Add the mount to the index.
     mByHandle[info.mHandle].emplace(mount);
-    mByName[flags.mName] = mount;
-    mByPath[path] = mount;
+    mByName[name] = mount;
+    mByPath[mount->path()] = mount;
 
     // Release lock.
     lock.unlock();
@@ -768,19 +782,19 @@ catch (std::runtime_error& exception)
 {
     // Unexpected error while enabling mount.
     FUSEErrorF("Unable to enable mount %s: %s",
-               path.toPath(false).c_str(),
+               name.c_str(),
                exception.what());
 
     return MOUNT_UNEXPECTED;
 }
 
-bool MountDB::enabled(const LocalPath& path) const
+bool MountDB::enabled(const std::string& name) const
 {
     // Acquire lock.
     MountDBLock guard(*this);
 
     // Is a mount with this path enabled?
-    return mByPath.count(path) > 0;
+    return mByName.count(name) > 0;
 }
 
 void MountDB::executorFlags(const TaskExecutorFlags& flags)
@@ -795,7 +809,7 @@ TaskExecutorFlags MountDB::executorFlags() const
     return mContext.serviceFlags().mMountExecutorFlags;
 }
 
-MountResult MountDB::flags(const LocalPath& path,
+MountResult MountDB::flags(const std::string& currentName,
                            const MountFlags& flags)
 try
 {
@@ -809,33 +823,31 @@ try
         return MOUNT_NO_NAME;
     }
 
-    // Is the mount enabled?
-    auto mount = this->mount(path);
+    auto transaction = mContext.mDatabase.transaction();
+    auto query = transaction.query(mQueries.mGetMountByName);
 
-    // Mount's enabled.
-    if (mount)
+    // Make sure the mount's new name, if any, is unique.
+    if (currentName != flags.mName)
     {
-        // Is this name already taken by another mount?
-        auto other = this->mount(flags.mName);
+        query.param(":name") = flags.mName;
+        query.execute();
 
-        // Name's taken by another enabled mount.
-        if (other && other != mount)
+        // Mount's new name isn't unique.
+        if (query)
         {
-            FUSEErrorF("Name %s already taken by mount: %s",
-                       flags.mName.c_str(),
-                       other->path().toPath(false).c_str());
+            FUSEErrorF("Name \"%s\" already taken by another mount.",
+                       flags.mName.c_str());
 
             return MOUNT_NAME_TAKEN;
         }
     }
 
     // Update the mount's flags.
-    auto transaction = mContext.mDatabase.transaction();
-    auto query = transaction.query(mQueries.mSetMountFlagsByPath);
+    query = transaction.query(mQueries.mSetMountFlagsByName);
 
     flags.serialize(query);
 
-    query.param(":path") = path;
+    query.param(":current_name") = currentName;
     query.execute();
 
     // No mount is associated with this path.
@@ -843,15 +855,13 @@ try
         return MOUNT_UNKNOWN;
 
     // Mount's enabled.
-    if (mount)
+    if (auto mount = this->mount(currentName))
     {
-        auto name = mount->name();
-
         // Keep mount consistent with the database.
         mount->flags(flags);
 
         // Keep the by-name index up to date.
-        mByName.erase(name);
+        mByName.erase(currentName);
         mByName.emplace(flags.mName, mount);
     }
 
@@ -864,26 +874,26 @@ catch (std::runtime_error& exception)
 {
     // Unexpected error while updating the mount's flags.
     FUSEErrorF("Unable to update flags for mount %s: %s",
-               path.toPath(false).c_str(),
+               currentName.c_str(),
                exception.what());
 
     return MOUNT_UNEXPECTED;
 }
 
-MountFlagsPtr MountDB::flags(const LocalPath& path) const
+MountFlagsPtr MountDB::flags(const std::string& name) const
 try
 {
     auto guard = lockAll(mContext.mDatabase, *this);
 
     // Fast path: Mount's enabled and in memory.
-    if (auto mount = this->mount(path))
+    if (auto mount = this->mount(name))
         return std::make_unique<MountFlags>(mount->flags());
 
     // Retrieve mount's flags from the database.
     auto transaction = mContext.mDatabase.transaction();
-    auto query = transaction.query(mQueries.mGetMountFlagsByPath);
+    auto query = transaction.query(mQueries.mGetMountFlagsByName);
 
-    query.param(":path") = path;
+    query.param(":name") = name;
     query.execute();
 
     // No mount's associated with the specified path.
@@ -900,26 +910,26 @@ catch (std::runtime_error& exception)
 {
     // Unexpected error while retrieving the mount's flags.
     FUSEErrorF("Unable to retrieve flags for mount %s: %s",
-               path.toPath(false).c_str(),
+               name.c_str(),
                exception.what());
 
     return nullptr;
 }
 
-MountInfoPtr MountDB::get(const LocalPath& path) const
+MountInfoPtr MountDB::get(const std::string& name) const
 try
 {
     auto guard = lockAll(mContext.mDatabase, *this);
 
     // Fast path: Mount's enabled and in memory.
-    if (auto mount = this->mount(path))
+    if (auto mount = this->mount(name))
         return std::make_unique<MountInfo>(mount->info());
 
     // Retrieve the mount's description from the database.
     auto transaction = mContext.mDatabase.transaction();
-    auto query = transaction.query(mQueries.mGetMountByPath);
+    auto query = transaction.query(mQueries.mGetMountByName);
 
-    query.param(":path") = path;
+    query.param(":name") = name;
     query.execute();
 
     // No mount associated with the specified path.
@@ -936,29 +946,35 @@ catch (std::runtime_error& exception)
 {
     // Unexpected error while retrieving the mount's description.
     FUSEErrorF("Unable to retrieve information for mount %s: %s",
-               path.toPath(false).c_str(),
+               name.c_str(),
                exception.what());
 
     return nullptr;
 }
 
-MountInfoVector MountDB::get(bool enabled) const
+MountInfoVector MountDB::get(bool onlyEnabled) const
 try
 {
     auto guard = lockAll(mContext.mDatabase, *this);
 
-    MountInfoVector mounts;
+    MountInfoSet<MountInfoNameLess> mounts;
+
+    // Copy descriptions of enabled mounts.
+    //
+    // Note that we do this even when onlyEnabled is false.
+    //
+    // The reason is that some mounts may only have a defined path when they
+    // are enabled and we want to provide that path to the caller when
+    // possible.
+    //
+    // We rely on the set above to prevent entries from the database from
+    // overwriting what we've retrieved from memory.
+    for (auto& i : mByPath)
+        mounts.emplace(i.second->info());
 
     // Caller's interested only in mounts that are enabled.
-    if (enabled)
-    {
-        // Copy descriptions of enabled mounts.
-        for (auto& i : mByPath)
-            mounts.emplace_back(i.second->info());
-
-        // And return them to the caller.
-        return mounts;
-    }
+    if (onlyEnabled)
+        return MountInfoVector(mounts.begin(), mounts.end());
 
     // Retrieve descriptions of all known mounts from the database.
     auto transaction = mContext.mDatabase.transaction();
@@ -968,10 +984,10 @@ try
 
     // Deserialize each description.
     for ( ; query; ++query)
-        mounts.emplace_back(MountInfo::deserialize(query));
+        mounts.emplace(MountInfo::deserialize(query));
 
     // And return them all to the caller.
-    return mounts;
+    return MountInfoVector(mounts.begin(), mounts.end());
 }
 catch (std::runtime_error& exception)
 {
@@ -982,22 +998,24 @@ catch (std::runtime_error& exception)
     return MountInfoVector();
 }
 
-NormalizedPathVector MountDB::paths(const std::string& name) const
+NormalizedPath MountDB::path(const std::string& name) const
 try
 {
     auto guard = lockAll(mContext.mDatabase, *this);
 
-    auto paths = NormalizedPathVector();
+    if (auto mount = this->mount(name))
+        return mount->path();
+
     auto transaction = mContext.mDatabase.transaction();
-    auto query = transaction.query(mQueries.mGetMountPathsByName);
+    auto query = transaction.query(mQueries.mGetMountPathByName);
 
     query.param(":name") = name;
     query.execute();
 
-    for ( ; query; ++query)
-        paths.emplace_back(query.field("path"));
+    if (query && !query.field("path").null())
+        return query.field("path").path();
 
-    return paths;
+    return NormalizedPath();
 }
 catch (std::runtime_error& exception)
 {
@@ -1005,7 +1023,7 @@ catch (std::runtime_error& exception)
                name.c_str(),
                exception.what());
 
-    return NormalizedPathVector();
+    return NormalizedPath();
 }
 
 MountResult MountDB::prune()
@@ -1028,23 +1046,22 @@ catch (std::runtime_error& exception)
     return MOUNT_UNEXPECTED;
 }
 
-MountResult MountDB::remove(const LocalPath& path)
+MountResult MountDB::remove(const std::string& name)
 try
 {
     auto guard = lockAll(mContext.mDatabase, *this);
 
-    if (mByPath.count(path))
+    if (mByName.count(name))
     {
-        FUSEErrorF("Can't remove an enabled mount: %s",
-                   path.toPath(false).c_str());
+        FUSEErrorF("Can't remove an enabled mount: %s", name.c_str());
 
         return MOUNT_BUSY;
     }
 
     auto transaction = mContext.mDatabase.transaction();
-    auto query = transaction.query(mQueries.mRemoveMountByPath);
+    auto query = transaction.query(mQueries.mRemoveMountByName);
 
-    query.param(":path") = path;
+    query.param(":name") = name;
     query.execute();
 
     transaction.commit();
@@ -1054,7 +1071,7 @@ try
 catch (std::runtime_error& exception)
 {
     FUSEErrorF("Unable to remove mount %s: %s",
-               path.toPath(false).c_str(),
+               name.c_str(),
                exception.what());
 
     return MOUNT_UNEXPECTED;
