@@ -19,13 +19,14 @@ class Unmounter final
 {
     // Report the result of an unmount operation.
     void emitEvent(MountDisabledCallback callback,
-                   LocalPath path,
+                   const std::string& name,
                    MountResult result);
 
     // Try and unmount the specified mount.
     void unmount(MountDisabledCallback callback,
                  MountWeakPtr mount,
-                 LocalPath path);
+                 const std::string& name,
+                 const LocalPath& path);
 
     // Unmount the specified mount.
     MountResult unmount(Mount& mount,
