@@ -782,6 +782,7 @@ Mount::Mount(const MountInfo& info, MountDB& mountDB)
   : fuse::Mount(info, mountDB)
   , mActivities()
   , mExecutor(mountDB.executorFlags())
+  , mPath(info.mPath)
   , mSession(*this)
   , mInvalidator(mSession)
 {
@@ -868,6 +869,11 @@ MountInodeID Mount::map(InodeID id) const
     return MountInodeID(id);
 }
 
+NormalizedPath Mount::path() const
+{
+    return mPath;
+}
+
 } // platform
 } // fuse
 } // mega
