@@ -154,7 +154,7 @@ NTSTATUS Mount::create(const std::wstring& path,
     // Couldn't create the new node.
     if (!created)
         return translate(created.error());
-    
+
     // Latch the new node's description.
     translate(info, *this, std::get<1>(*created));
 
@@ -830,10 +830,10 @@ Mount::Mount(const MountInfo& info,
              MountDB& mountDB)
   : fuse::Mount(info, mountDB)
   , mActivities()
-  , mDispatcher(*this)
+  , mDispatcher(*this, info.mPath)
   , mExecutor(mountDB.executorFlags())
 {
-    mDispatcher.start();
+    mDispatcher.start(info.mPath);
 
     FUSEDebugF("Mount constructed: %s",
                path().toPath(false).c_str());
@@ -871,6 +871,11 @@ MountInodeID Mount::map(InodeID id) const
     return MountInodeID(id);
 }
 
+NormalizedPath Mount::path() const
+{
+    return mDispatcher.path();
+}
+
 MountResult Mount::remove()
 {
     // Remove the mount from memory.
