@@ -185,7 +185,7 @@ Proxy *HttpIO::getautoproxy()
                 }
             }
 
-            proxy->setProxyURL(&proxyURL);
+            proxy->setProxyURL(proxyURL);
         }
         else if (ieProxyConfig.lpszAutoConfigUrl || ieProxyConfig.fAutoDetect == TRUE)
         {
@@ -223,7 +223,7 @@ Proxy *HttpIO::getautoproxy()
                     string proxyURL;
                     proxy->setProxyType(Proxy::CUSTOM);
                     proxyURL.assign((const char*)proxyInfo.lpszProxy, wcslen(proxyInfo.lpszProxy) * sizeof(wchar_t));
-                    proxy->setProxyURL(&proxyURL);
+                    proxy->setProxyURL(proxyURL);
                 }
             }
             WinHttpCloseHandle(hSession);
@@ -253,73 +253,6 @@ Proxy *HttpIO::getautoproxy()
     return proxy;
 }
 
-void HttpIO::getMEGADNSservers(string* dnsservers, bool getfromnetwork)
-{
-    if (!dnsservers)
-    {
-        return;
-    }
-
-    dnsservers->clear();
-    if (getfromnetwork)
-    {
-        struct addrinfo* aiList = NULL;
-        struct addrinfo* hp;
-
-        struct addrinfo hints = {};
-        hints.ai_family = AF_UNSPEC;
-#ifdef AI_V4MAPPED
-        hints.ai_flags |= AI_V4MAPPED;
-#endif
-#ifdef AI_ADDRCONFIG
-        hints.ai_flags |= AI_ADDRCONFIG;
-#endif
-
-        if (!getaddrinfo("ns.mega.co.nz", NULL, &hints, &aiList))
-        {
-            hp = aiList;
-            while (hp)
-            {
-                char straddr[INET6_ADDRSTRLEN];
-                straddr[0] = 0;
-
-                if (hp->ai_family == AF_INET)
-                {
-                    sockaddr_in* addr = (sockaddr_in*)hp->ai_addr;
-                    mega_inet_ntop(hp->ai_family, &addr->sin_addr, straddr, sizeof(straddr));
-                }
-                else if (hp->ai_family == AF_INET6)
-                {
-                    sockaddr_in6* addr = (sockaddr_in6*)hp->ai_addr;
-                    mega_inet_ntop(hp->ai_family, &addr->sin6_addr, straddr, sizeof(straddr));
-                }
-
-                if (straddr[0])
-                {
-                    if (dnsservers->size())
-                    {
-                        dnsservers->append(",");
-                    }
-                    dnsservers->append(straddr);
-                }
-
-                hp = hp->ai_next;
-            }
-            freeaddrinfo(aiList);
-        }
-    }
-
-    if (!getfromnetwork || !dnsservers->size())
-    {
-        *dnsservers = DNS_SERVERS;
-        LOG_info << "Using hardcoded MEGA DNS servers: " << *dnsservers;
-    }
-    else
-    {
-        LOG_info << "Using current MEGA DNS servers: " << *dnsservers;
-    }
-}
-
 bool HttpIO::setmaxdownloadspeed(m_off_t)
 {
     return false;
@@ -340,6 +273,13 @@ m_off_t HttpIO::getmaxuploadspeed()
     return 0;
 }
 
+void HttpIO::setproxy(const Proxy&) {}
+
+std::optional<Proxy> HttpIO::getproxy() const
+{
+    return std::nullopt;
+}
+
 void HttpReq::prepareMethod(HttpIO* clientHttpIo, const httpmethod_t reqMethod)
 {
     if (httpio)
