@@ -681,37 +681,50 @@ uint64_t NodeManager::getNodeCount_internal()
     return count;
 }
 
-std::set<std::string> NodeManager::getAllNodeTags(const char* searchString, CancelToken cancelFlag)
+auto NodeManager::getNodeTagsBelow(CancelToken cancelToken,
+                                   const std::set<NodeHandle>& handles,
+                                   const std::string& pattern)
+    -> std::optional<std::set<std::string>>
 {
-    LockGuard g(mMutex);
-    return getAllNodeTags_internal(searchString, cancelFlag);
-}
+    // Make sure we have exclusive access to the database.
+    LockGuard guard(mMutex);
 
-std::set<std::string> NodeManager::getAllNodeTags_internal(const char* searchString,
-                                                           CancelToken cancelFlag)
-{
-    assert(mMutex.owns_lock());
-    // validation
+    // Convenience.
+    static const auto warning = [](const char* message)
+    {
+        LOG_warn << "getNodeTagsBelow: " << message;
+        return std::nullopt;
+    }; // warning
+
+    // Make sure the database is sane and that some nodes exist.
     if (!mTable || mNodes.empty())
-    {
-        LOG_warn
-            << "getAllNodeTags_internal: The database is not opened or there are no nodes loaded";
-        assert(mTable && !mNodes.empty());
-        return {};
-    }
-    const std::string auxSearchString = searchString ? searchString : "";
-    // ',' cannot be in the string
-    if (auxSearchString.find(MegaClient::TAG_DELIMITER) != std::string::npos)
-    {
-        LOG_warn << "getAllNodeTags_internal: The search string (" << auxSearchString
-                 << ") contains an invalid character (,)";
-        return {};
-    }
-    std::set<std::string> result;
-    if (!mTable->getAllNodeTags(auxSearchString, result, cancelFlag))
-        return {};
+        return warning("The database hasn't been opened or there are no nodes present");
 
-    return result;
+    // Make sure the caller isn't trying to filter by multiple tags simultaneously.
+    if (pattern.find(MegaClient::TAG_DELIMITER) != std::string::npos)
+        return warning("You can't filter by multiple tags at the same time");
+
+    std::optional<std::set<std::string>> accumulatedTags;
+
+    // Try and retrieve the tags below the specified nodes.
+    for (const auto& handle: handles)
+    {
+        // Try and retrieve tags below this node.
+        auto tags = mTable->getNodeTagsBelow(cancelToken, handle, pattern);
+
+        // Couldn't get tags.
+        if (!tags)
+            continue;
+
+        // Merge tags into accumulated result if possible.
+        if (accumulatedTags)
+            accumulatedTags->merge(*tags);
+        else
+            accumulatedTags = std::move(tags);
+    }
+
+    // Return accumulated tags to caller.
+    return accumulatedTags;
 }
 
 sharedNode_vector NodeManager::searchNodes(const NodeSearchFilter& filter, int order, CancelToken cancelFlag, const NodeSearchPage& page)
@@ -782,13 +795,13 @@ sharedNode_vector NodeManager::getNodesWithLinks()
     return getNodesWithSharesOrLink_internal(ShareType_t::LINK);
 }
 
-sharedNode_vector NodeManager::getNodesByFingerprint(FileFingerprint &fingerprint)
+sharedNode_vector NodeManager::getNodesByFingerprint(const FileFingerprint& fingerprint)
 {
     LockGuard g(mMutex);
     return getNodesByFingerprint_internal(fingerprint);
 }
 
-sharedNode_vector NodeManager::getNodesByFingerprint_internal(FileFingerprint &fingerprint)
+sharedNode_vector NodeManager::getNodesByFingerprint_internal(const FileFingerprint& fingerprint)
 {
     assert(mMutex.owns_lock());
 
@@ -804,7 +817,7 @@ sharedNode_vector NodeManager::getNodesByFingerprint_internal(FileFingerprint &f
     auto p = mFingerPrints.equal_range(&fingerprint);
     for (auto it = p.first; it != p.second; ++it)
     {
-        Node* node = static_cast<Node*>(*it);
+        const auto node = static_cast<const Node*>(*it);
         fpLoaded.emplace(node->nodeHandle());
         std::shared_ptr<Node> sharedNode = node->mNodePosition->second.getNodeInRam();
         assert(sharedNode && "Node loaded at fingerprint map should have a node in RAM ");
@@ -899,7 +912,7 @@ std::shared_ptr<Node> NodeManager::getNodeByFingerprint_internal(FileFingerprint
     auto it = mFingerPrints.find(&fingerprint);
     if (it != mFingerPrints.end())
     {
-        Node *n = static_cast<Node*>(*it);
+        const auto n = static_cast<const Node*>(*it);
         assert(n);
         return n->mNodePosition->second.getNodeInRam();
     }
