@@ -39,8 +39,11 @@
 #endif
 
 #ifdef __ANDROID__
+#include <mega/android/androidFileSystem.h>
+
 #include <jni.h>
 extern JavaVM *MEGAjvm;
+extern jclass fileWrapper;
 #endif
 
 #if defined(__MACH__) && !(TARGET_OS_IPHONE)
@@ -187,7 +190,7 @@ auto LocalPath::asPlatformEncoded(bool) const -> string_type
 }
 bool LocalPath::isRootPath() const
 {
-    if (isFromRoot)
+    if (isAbsolute())
         return localpath.size() == 1 && localpath.back() == '/';
 
     return false;
@@ -2358,9 +2361,6 @@ fsfp_t FileSystemAccess::fsFingerprint(const LocalPath& path) const
             return fsfp_t(fingerprint, std::move(uuid));
     }
 
-    LOG_warn << "Falling back to legacy filesystem fingerprint: "
-             << path;
-
     // Couldn't determine filesystem UUID.
     return fsfp_t(fingerprint, std::string());
 }
@@ -2411,12 +2411,36 @@ bool PosixFileSystemAccess::hardLink(const LocalPath& source, const LocalPath& t
 
 std::unique_ptr<FileAccess> PosixFileSystemAccess::newfileaccess(bool followSymLinks)
 {
+#ifndef __ANDROID__
     return std::unique_ptr<FileAccess>{new PosixFileAccess{waiter, defaultfilepermissions, followSymLinks}};
+#else
+    if (fileWrapper != nullptr)
+    {
+        return std::unique_ptr<FileAccess>{
+            new AndroidFileAccess{waiter, defaultfilepermissions, followSymLinks}};
+    }
+    else
+    {
+        return std::unique_ptr<FileAccess>{
+            new PosixFileAccess{waiter, defaultfilepermissions, followSymLinks}};
+    }
+#endif
 }
 
 unique_ptr<DirAccess>  PosixFileSystemAccess::newdiraccess()
 {
+#ifndef __ANDROID__
     return unique_ptr<DirAccess>(new PosixDirAccess());
+#else
+    if (fileWrapper != nullptr)
+    {
+        return unique_ptr<DirAccess>(new AndroidDirAccess());
+    }
+    else
+    {
+        return unique_ptr<DirAccess>(new PosixDirAccess());
+    }
+#endif
 }
 
 #ifdef __linux__
