@@ -35,40 +35,63 @@ void Proxy::setProxyType(int newType)
     proxyType = newType;
 }
 
-void Proxy::setProxyURL(string* newURL)
+void Proxy::setProxyURL(const string& newURL)
 {
-    proxyURL = *newURL;
+    proxyURL = newURL;
 }
 
-void Proxy::setCredentials(string* newUsername, string* newPassword)
+void Proxy::setCredentials(const string& newUsername, const string& newPassword)
 {
-    username = *newUsername;
-    password = *newPassword;
+    username = newUsername;
+    password = newPassword;
 }
 
-int Proxy::getProxyType()
+int Proxy::getProxyType() const
 {
     return proxyType;
 }
 
-string Proxy::getProxyURL()
+string Proxy::getProxyURL() const
 {
     return this->proxyURL;
 }
 
-bool Proxy::credentialsNeeded()
+bool Proxy::credentialsNeeded() const
 {
     return (username.size() != 0);
 }
 
-string Proxy::getUsername()
+string Proxy::getUsername() const
 {
     return username;
 }
 
-string Proxy::getPassword()
+string Proxy::getPassword() const
 {
     return password;
 }
 
+int proxyTypeFromString(const std::string& type)
+{
+#define ENTRY(name) {#name, Proxy::name},
+    static const std::map<std::string, int> types = {DEFINE_PROXY_TYPES(ENTRY)}; // types
+#undef ENTRY
+
+    if (auto i = types.find(type); i != types.end())
+        return i->second;
+
+    return NONE;
+}
+
+const std::string* proxyTypeToString(int type)
+{
+#define ENTRY(name) {Proxy::name, #name},
+    static const std::map<int, std::string> strings = {DEFINE_PROXY_TYPES(ENTRY)}; // strings
+#undef ENTRY
+
+    if (auto i = strings.find(type); i != strings.end())
+        return &i->second;
+
+    return nullptr;
+}
 }
