@@ -114,7 +114,7 @@ auto LocalPath::asPlatformEncoded(bool skipPrefix) const -> string_type
 
 bool LocalPath::isRootPath() const
 {
-    if (!isFromRoot)
+    if (!isAbsolute())
         return false;
 
     static const std::wstring prefix = L"\\\\?\\";
@@ -841,9 +841,11 @@ bool WinFileAccess::fopen_impl(const LocalPath& namePath, bool read, bool write,
 
     mtime = FileTime_to_POSIX(&fad.ftLastWriteTime);
 
-    if (!write && (fsidvalid = !!GetFileInformationByHandle(hFile, &bhfi)))
+    if (!write)
     {
-        fsid = ((handle)bhfi.nFileIndexHigh << 32) | (handle)bhfi.nFileIndexLow;
+        fsidvalid = GetFileInformationByHandle(hFile, &bhfi) != FALSE;
+        if (fsidvalid)
+            fsid = ((handle)bhfi.nFileIndexHigh << 32) | (handle)bhfi.nFileIndexLow;
     }
 
     if (type == FOLDERNODE)
@@ -977,8 +979,12 @@ bool WinFileSystemAccess::getsname(const LocalPath& namePath, LocalPath& snamePa
     // we are only interested in the path's last component
     wchar_t* ptr;
 
-    if ((ptr = wcsrchr(const_cast<wchar_t*>(sname.data()), L'\\')) ||
-        (ptr = wcsrchr(const_cast<wchar_t*>(sname.data()), L':')))
+    ptr = wcsrchr(const_cast<wchar_t*>(sname.data()), L'\\');
+    if (!ptr)
+    {
+        ptr = wcsrchr(const_cast<wchar_t*>(sname.data()), L':');
+    }
+    if (ptr)
     {
         sname.erase(0, ptr - sname.data() + 1);
     }
@@ -1261,7 +1267,7 @@ bool WinFileSystemAccess::expanselocalpath(const LocalPath& pathArg, LocalPath&
         }
     }
 
-    absolutepathArg.isFromRoot = true;
+    absolutepathArg.mPathType = PathType::ABSOLUTE_PATH;
     return true;
 }
 
@@ -2154,7 +2160,8 @@ bool WinDirAccess::dopen(LocalPath* nameArg, FileAccess* f, bool glob)
         }
     }
 
-    if (!(ffdvalid = hFind != INVALID_HANDLE_VALUE))
+    ffdvalid = hFind != INVALID_HANDLE_VALUE;
+    if (!ffdvalid)
     {
         return false;
     }
@@ -2174,7 +2181,7 @@ bool WinDirAccess::dnext(LocalPath& /*path*/, LocalPath& nameArg, bool /*follows
           || (ffd.cFileName[1] && ((ffd.cFileName[1] != '.') || ffd.cFileName[2]))))
         {
             nameArg.localpath.assign(ffd.cFileName, wcslen(ffd.cFileName));
-            nameArg.isFromRoot = false;
+            nameArg.mPathType = PathType::RELATIVE_PATH;
 
             if (!globbase.empty())
             {
@@ -2199,7 +2206,8 @@ bool WinDirAccess::dnext(LocalPath& /*path*/, LocalPath& nameArg, bool /*follows
             }
         }
 
-        if (!(ffdvalid = FindNextFileW(hFind, &ffd) != 0))
+        ffdvalid = FindNextFileW(hFind, &ffd) != FALSE;
+        if (!ffdvalid)
         {
             return false;
         }
