@@ -23,6 +23,7 @@
 #include "../include/megaapi_impl.h"
 #include "gtest/gtest.h"
 #include "mega.h"
+#include "sdk_test_data_provider.h"
 #include "test.h"
 
 #include <atomic>
@@ -30,6 +31,7 @@
 #include <future>
 #include <iostream>
 #include <memory>
+#include <type_traits>
 
 #ifndef WIN32
 #define DOTSLASH "./"
@@ -47,6 +49,7 @@ using ::testing::Test;
 static const unsigned int pollingT      = 500000;   // (microseconds) to check if response from server is received
 static const unsigned int maxTimeout    = 600;      // Maximum time (seconds) to wait for response from server
 static const unsigned int defaultTimeout = 60;      // Normal time for most operations (seconds) to wait for response from server
+static const unsigned int defaultTimeoutMs = defaultTimeout * 1000;
 static const unsigned int waitForSyncsMs = 4000;    // Time to wait after a sync has been created and before adding new files to it
 
 /**
@@ -429,11 +432,55 @@ private:
     std::string mEndpointName;
 };
 
+// Poor man's expected.
+template<typename T>
+using Expected = std::variant<Error, T>;
+
+template<typename T>
+struct IsExpected: std::false_type
+{}; // IsExpected<T>
+
+template<typename T>
+struct IsExpected<Expected<T>>: std::true_type
+{}; // IsExpected<Expected<T>>
+
+template<typename T>
+static constexpr auto IsExpectedV = IsExpected<T>::value;
+
+template<typename T>
+using RemoveCVRef = std::remove_cv<std::remove_reference_t<T>>;
+
+template<typename T>
+using RemoveCVRefT = typename RemoveCVRef<T>::type;
+
+template<typename T>
+Error result(const Expected<T>& expected)
+{
+    if (auto* result = std::get_if<0>(&expected))
+        return *result;
+
+    return API_OK;
+}
+
+template<typename T, typename = std::enable_if_t<IsExpectedV<RemoveCVRefT<T>>>>
+decltype(auto) value(T&& expected)
+{
+    assert(result(expected) == API_OK);
+
+    return std::get<1>(std::forward<T>(expected));
+}
+
 using MegaApiTestPointer = std::unique_ptr<MegaApiTest, MegaApiTestDeleter>;
 
 // Fixture class with common code for most of tests
-class SdkTest : public SdkTestBase, public MegaListener, public MegaRequestListener, MegaTransferListener, MegaLogger {
-
+class SdkTest:
+    public SdkTestBase,
+    public SdkTestDataProvider,
+    public MegaListener,
+    public MegaRequestListener,
+    MegaTransferListener,
+    MegaLogger
+{
 public:
     struct PerApi
     {
@@ -622,7 +669,7 @@ protected:
     void testRecents(const std::string& title, bool useSensitiveExclusion);
 
 #ifdef ENABLE_CHAT
-    void delSchedMeetings();
+    void cancelSchedMeetings();
 #endif
 
     void syncTestEnsureMyBackupsRemoteFolderExists(unsigned apiIdx);
@@ -646,8 +693,6 @@ protected:
     void onSetElementsUpdate(MegaApi *api, MegaSetElementList *elements) override;
     void onContactRequestsUpdate(MegaApi* api, MegaContactRequestList* requests) override;
 
-    void onReloadNeeded(MegaApi*) override {}
-
     void onUserAlertsUpdate(MegaApi* api, MegaUserAlertList* alerts) override;
 
 #ifdef ENABLE_SYNC
@@ -693,12 +738,65 @@ public:
     // *** WARNING *** THESE FUNCTIONS RETURN VALUE ARE SUBJECT TO RACE CONDITIONS
     // convenience functions - template args just make it easy to code, no need to copy all the exact argument types with listener defaults etc. To add a new one, just copy a line and change the flag and the function called.
     // WARNING: any sort of race can result in the lastError being set from some other command - better to use the listener based ones in the next list below
-    template<typename ... Args> int synchronousCatchup(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_CATCHUP, [this, apiIndex, args...]() { megaApi[apiIndex]->catchup(args...); }); return mApi[apiIndex].lastError; }
-    template<typename ... Args> int synchronousCreateEphemeralAccountPlusPlus(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_CREATE_ACCOUNT, [this, apiIndex, args...]() { megaApi[apiIndex]->createEphemeralAccountPlusPlus(args...); }); return mApi[apiIndex].lastError; }
-    template<typename ... Args> int synchronousResumeCreateAccountEphemeralPlusPlus(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_CREATE_ACCOUNT, [this, apiIndex, args...]() { megaApi[apiIndex]->resumeCreateAccountEphemeralPlusPlus(args...); }); return mApi[apiIndex].lastError; }
-    template<typename ... Args> int synchronousCreateAccount(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_CREATE_ACCOUNT, [this, apiIndex, args...]() { megaApi[apiIndex]->createAccount(args...); }); return mApi[apiIndex].lastError; }
-    template<typename ... Args> int synchronousResumeCreateAccount(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_CREATE_ACCOUNT, [this, apiIndex, args...]() { megaApi[apiIndex]->resumeCreateAccount(args...); }); return mApi[apiIndex].lastError; }
-    template<typename ... Args> int synchronousSendSignupLink(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_SEND_SIGNUP_LINK, [this, apiIndex, args...]() { megaApi[apiIndex]->sendSignupLink(args...); }); return mApi[apiIndex].lastError; }
+    template<typename... Args>
+    int synchronousCatchup(unsigned apiIndex, Args... args)
+    {
+        synchronousRequest(apiIndex,
+                           MegaRequest::TYPE_CATCHUP,
+                           [this, apiIndex, args...]()
+                           {
+                               megaApi[apiIndex]->catchup(args...);
+                           });
+        return mApi[apiIndex].lastError;
+    }
+
+    template<typename... Args>
+    int synchronousCreateEphemeralAccountPlusPlus(unsigned apiIndex, Args... args)
+    {
+        synchronousRequest(apiIndex,
+                           MegaRequest::TYPE_CREATE_ACCOUNT,
+                           [this, apiIndex, args...]()
+                           {
+                               megaApi[apiIndex]->createEphemeralAccountPlusPlus(args...);
+                           });
+        return mApi[apiIndex].lastError;
+    }
+
+    template<typename... Args>
+    int synchronousResumeCreateAccountEphemeralPlusPlus(unsigned apiIndex, Args... args)
+    {
+        synchronousRequest(apiIndex,
+                           MegaRequest::TYPE_CREATE_ACCOUNT,
+                           [this, apiIndex, args...]()
+                           {
+                               megaApi[apiIndex]->resumeCreateAccountEphemeralPlusPlus(args...);
+                           });
+        return mApi[apiIndex].lastError;
+    }
+
+    template<typename... Args>
+    int synchronousCreateAccount(unsigned apiIndex, Args... args)
+    {
+        synchronousRequest(apiIndex,
+                           MegaRequest::TYPE_CREATE_ACCOUNT,
+                           [this, apiIndex, args...]()
+                           {
+                               megaApi[apiIndex]->createAccount(args...);
+                           });
+        return mApi[apiIndex].lastError;
+    }
+
+    template<typename... Args>
+    int synchronousResumeCreateAccount(unsigned apiIndex, Args... args)
+    {
+        synchronousRequest(apiIndex,
+                           MegaRequest::TYPE_CREATE_ACCOUNT,
+                           [this, apiIndex, args...]()
+                           {
+                               megaApi[apiIndex]->resumeCreateAccount(args...);
+                           });
+        return mApi[apiIndex].lastError;
+    }
     template<typename ... Args> int synchronousConfirmSignupLink(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_CONFIRM_ACCOUNT, [this, apiIndex, args...]() { megaApi[apiIndex]->confirmAccount(args...); }); return mApi[apiIndex].lastError; }
     template<typename ... Args> int synchronousFastLogin(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_LOGIN, [this, apiIndex, args...]() { megaApi[apiIndex]->fastLogin(args...); }); return mApi[apiIndex].lastError; }
 
@@ -709,9 +807,29 @@ public:
     // apiIndex, args...]() { megaApi[apiIndex]->getCountryCallingCodes(args...); }); return
     // mApi[apiIndex].lastError; }
 
-    template<typename ... Args> int synchronousGetUserAvatar(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_GET_ATTR_USER, [this, apiIndex, args...]() { megaApi[apiIndex]->getUserAvatar(args...); }); return mApi[apiIndex].lastError; }
-    template<typename ... Args> int synchronousGetUserAttribute(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_GET_ATTR_USER, [this, apiIndex, args...]() { megaApi[apiIndex]->getUserAttribute(args...); }); return mApi[apiIndex].lastError; }
-    template<typename ... Args> int synchronousSetNodeDuration(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_SET_ATTR_NODE, [this, apiIndex, args...]() { megaApi[apiIndex]->setNodeDuration(args...); }); return mApi[apiIndex].lastError; }
+    template<typename... Args>
+    int synchronousGetUserAvatar(unsigned apiIndex, Args... args)
+    {
+        synchronousRequest(apiIndex,
+                           MegaRequest::TYPE_GET_ATTR_USER,
+                           [this, apiIndex, args...]()
+                           {
+                               megaApi[apiIndex]->getUserAvatar(args...);
+                           });
+        return mApi[apiIndex].lastError;
+    }
+
+    template<typename... Args>
+    int synchronousGetUserAttribute(unsigned apiIndex, Args... args)
+    {
+        synchronousRequest(apiIndex,
+                           MegaRequest::TYPE_GET_ATTR_USER,
+                           [this, apiIndex, args...]()
+                           {
+                               megaApi[apiIndex]->getUserAttribute(args...);
+                           });
+        return mApi[apiIndex].lastError;
+    }
     template<typename ... Args> int synchronousSetNodeCoordinates(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_SET_ATTR_NODE, [this, apiIndex, args...]() { megaApi[apiIndex]->setNodeCoordinates(args...); }); return mApi[apiIndex].lastError; }
     template<typename ... Args> int synchronousGetSpecificAccountDetails(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_ACCOUNT_DETAILS, [this, apiIndex, args...]() { megaApi[apiIndex]->getSpecificAccountDetails(args...); }); return mApi[apiIndex].lastError; }
     template<typename ... Args> int synchronousMediaUploadRequestURL(unsigned apiIndex, Args... args) { synchronousRequest(apiIndex, MegaRequest::TYPE_GET_BACKGROUND_UPLOAD_URL, [this, apiIndex, args...]() { megaApi[apiIndex]->backgroundMediaUploadRequestUploadURL(args...); }); return mApi[apiIndex].lastError; }
@@ -773,20 +891,118 @@ public:
         megaApi[static_cast<size_t>(apiIndex)]->fastLogin(args..., rt.get());
         return rt;
     }
-    template<typename ... requestArgs> std::unique_ptr<RequestTracker> asyncRequestFastLogin(MegaApi *api, requestArgs... args) { auto rt = std::make_unique<RequestTracker>(api); api->fastLogin(args..., rt.get()); return rt; }
-    template<typename ... requestArgs> std::unique_ptr<RequestTracker> asyncRequestLoginToFolder(unsigned apiIndex, requestArgs... args) { auto rt = std::make_unique<RequestTracker>(megaApi[apiIndex].get()); megaApi[apiIndex]->loginToFolder(args..., rt.get()); return rt; }
-    template<typename ... requestArgs> std::unique_ptr<RequestTracker> asyncRequestLoginToFolder(MegaApi *api, requestArgs... args) { auto rt = std::make_unique<RequestTracker>(api); api->loginToFolder(args..., rt.get()); return rt; }
-    template<typename ... requestArgs> std::unique_ptr<RequestTracker> asyncRequestLocalLogout(MegaApi *api, requestArgs... args) { auto rt = std::make_unique<RequestTracker>(api); api->localLogout(args..., rt.get()); return rt; }
-    template<typename ... requestArgs> std::unique_ptr<RequestTracker> asyncRequestFetchnodes(unsigned apiIndex, requestArgs... args) { auto rt = std::make_unique<RequestTracker>(megaApi[apiIndex].get()); megaApi[apiIndex]->fetchNodes(args..., rt.get()); return rt; }
-    template<typename ... requestArgs> std::unique_ptr<RequestTracker> asyncRequestFetchnodes(MegaApi *api, requestArgs... args) { auto rt = std::make_unique<RequestTracker>(api); api->fetchNodes(args..., rt.get()); return rt; }
-    template<typename ... requestArgs> std::unique_ptr<RequestTracker> asyncRequestGetVisibleWelcomeDialog(unsigned apiIndex) { auto rt = std::make_unique<RequestTracker>(megaApi[apiIndex].get()); megaApi[apiIndex]->getVisibleWelcomeDialog(rt.get()); return rt; }
-    template<typename ... requestArgs> int doGetDeviceName(unsigned apiIndex, string* dvc, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->getDeviceName(args..., &rt); auto e = rt.waitForResult(); if (dvc && e == API_OK) *dvc = rt.request->getName(); return e; }
-    template<typename ... requestArgs> int doSetDeviceName(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->setDeviceName(args..., &rt); return rt.waitForResult(); }
-    template<typename ... requestArgs> int doGetDriveName(unsigned apiIndex, string* drv, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->getDriveName(args..., &rt); auto e = rt.waitForResult(); if (drv && e == API_OK) *drv = rt.request->getName(); return e; }
-    template<typename ... requestArgs> int doSetDriveName(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->setDriveName(args..., &rt); return rt.waitForResult(); }
-    template<typename ... requestArgs> int doRequestLogout(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->logout(args..., &rt); return rt.waitForResult(); }
-    template<typename ... requestArgs> int doRequestLocalLogout(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->localLogout(args..., &rt); return rt.waitForResult(); }
-    template<typename ... requestArgs> int doSetNodeDuration(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->setNodeDuration(args..., &rt); return rt.waitForResult(); }
+
+    template<typename... requestArgs>
+    std::unique_ptr<RequestTracker> asyncRequestFastLogin(MegaApi* api, requestArgs... args)
+    {
+        auto rt = std::make_unique<RequestTracker>(api);
+        api->fastLogin(args..., rt.get());
+        return rt;
+    }
+
+    template<typename... requestArgs>
+    std::unique_ptr<RequestTracker> asyncRequestLoginToFolder(unsigned apiIndex,
+                                                              requestArgs... args)
+    {
+        auto rt = std::make_unique<RequestTracker>(megaApi[apiIndex].get());
+        megaApi[apiIndex]->loginToFolder(args..., rt.get());
+        return rt;
+    }
+
+    template<typename... requestArgs>
+    std::unique_ptr<RequestTracker> asyncRequestLoginToFolder(MegaApi* api, requestArgs... args)
+    {
+        auto rt = std::make_unique<RequestTracker>(api);
+        api->loginToFolder(args..., rt.get());
+        return rt;
+    }
+
+    template<typename... requestArgs>
+    std::unique_ptr<RequestTracker> asyncRequestLocalLogout(MegaApi* api, requestArgs... args)
+    {
+        auto rt = std::make_unique<RequestTracker>(api);
+        api->localLogout(args..., rt.get());
+        return rt;
+    }
+
+    template<typename... requestArgs>
+    std::unique_ptr<RequestTracker> asyncRequestFetchnodes(unsigned apiIndex, requestArgs... args)
+    {
+        auto rt = std::make_unique<RequestTracker>(megaApi[apiIndex].get());
+        megaApi[apiIndex]->fetchNodes(args..., rt.get());
+        return rt;
+    }
+
+    template<typename... requestArgs>
+    std::unique_ptr<RequestTracker> asyncRequestFetchnodes(MegaApi* api, requestArgs... args)
+    {
+        auto rt = std::make_unique<RequestTracker>(api);
+        api->fetchNodes(args..., rt.get());
+        return rt;
+    }
+
+    template<typename... requestArgs>
+    std::unique_ptr<RequestTracker> asyncRequestGetVisibleWelcomeDialog(unsigned apiIndex)
+    {
+        auto rt = std::make_unique<RequestTracker>(megaApi[apiIndex].get());
+        megaApi[apiIndex]->getVisibleWelcomeDialog(rt.get());
+        return rt;
+    }
+
+    template<typename... requestArgs>
+    int doGetDeviceName(unsigned apiIndex, string* dvc, requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->getDeviceName(args..., &rt);
+        auto e = rt.waitForResult();
+        if (dvc && e == API_OK)
+            *dvc = rt.request->getName();
+        return e;
+    }
+
+    template<typename... requestArgs>
+    int doSetDeviceName(unsigned apiIndex, requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->setDeviceName(args..., &rt);
+        return rt.waitForResult();
+    }
+
+    template<typename... requestArgs>
+    int doGetDriveName(unsigned apiIndex, string* drv, requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->getDriveName(args..., &rt);
+        auto e = rt.waitForResult();
+        if (drv && e == API_OK)
+            *drv = rt.request->getName();
+        return e;
+    }
+
+    template<typename... requestArgs>
+    int doSetDriveName(unsigned apiIndex, requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->setDriveName(args..., &rt);
+        return rt.waitForResult();
+    }
+
+    template<typename... requestArgs>
+    int doRequestLogout(unsigned apiIndex, requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->logout(args..., &rt);
+        return rt.waitForResult();
+    }
+
+    template<typename... requestArgs>
+    int doRequestLocalLogout(unsigned apiIndex, requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->localLogout(args..., &rt);
+        return rt.waitForResult();
+    }
+
     template<typename ... requestArgs> int doStartUpload(unsigned apiIndex, MegaHandle* newNodeHandleResult, requestArgs... args) { TransferTracker tt(megaApi[apiIndex].get()); megaApi[apiIndex]->startUpload(args..., &tt); auto e = tt.waitForResult(); if (newNodeHandleResult) *newNodeHandleResult = tt.resultNodeHandle; return e;}
     template<typename ... requestArgs> int doStartDownload(unsigned apiIndex, requestArgs... args) { TransferTracker tt(megaApi[apiIndex].get()); megaApi[apiIndex]->startDownload(args..., &tt); auto e = tt.waitForResult(); return e;}
     template<typename ... requestArgs> int doSetFileVersionsOption(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->setFileVersionsOption(args..., &rt); return rt.waitForResult(); }
@@ -902,9 +1118,22 @@ public:
         if (err == API_OK) recommendedLevel = (int)rt.request->getNumber();
         return err;
     }
-    template<typename ... requestArgs> int synchronousChangeEmail(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->changeEmail(args..., &rt); return rt.waitForResult(); }
-    template<typename ... requestArgs> int synchronousConfirmChangeEmail(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->confirmChangeEmail(args..., &rt); return rt.waitForResult(); }
-    template<typename ... requestArgs> int syncSendABTestActive(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->sendABTestActive(args..., &rt); return rt.waitForResult(); }
+
+    template<typename... requestArgs>
+    int synchronousChangeEmail(unsigned apiIndex, requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->changeEmail(args..., &rt);
+        return rt.waitForResult();
+    }
+
+    template<typename... requestArgs>
+    int synchronousConfirmChangeEmail(unsigned apiIndex, requestArgs... args)
+    {
+        RequestTracker rt(megaApi[apiIndex].get());
+        megaApi[apiIndex]->confirmChangeEmail(args..., &rt);
+        return rt.waitForResult();
+    }
 #ifdef ENABLE_SYNC
     template<typename ... requestArgs> int syncMoveToDebris(unsigned apiIndex, requestArgs... args) { RequestTracker rt(megaApi[apiIndex].get()); megaApi[apiIndex]->moveToDebris(args..., &rt); return rt.waitForResult(); }
 #endif // ENABLE_SYNC
@@ -932,7 +1161,7 @@ public:
 
     void inviteTestAccount(const unsigned invitorIndex, const unsigned inviteIndex, const string &message);
     void inviteContact(unsigned apiIndex, const string &email, const string& message, const int action);
-    void replyContact(MegaContactRequest *cr, int action);
+    void replyContact(MegaContactRequest* cr, int action, const unsigned apiIndex = 1);
     int removeContact(unsigned apiIndex, string email);
     void getUserAttribute(MegaUser *u, int type, int timeout = maxTimeout, int accountIndex = 1);
 
@@ -944,7 +1173,6 @@ public:
 #ifdef ENABLE_CHAT
     void createChatScheduledMeeting(const unsigned apiIndex, MegaHandle& chatid);
     void updateScheduledMeeting(const unsigned apiIndex, MegaHandle& chatid);
-    void deleteScheduledMeeting(unsigned apiIndex, MegaHandle& chatid);
 #endif
 
     string createPublicLink(unsigned apiIndex, MegaNode *n, m_time_t expireDate, int timeout, bool isFreeAccount, bool writable = false, bool megaHosted = false);
@@ -995,24 +1223,6 @@ public:
         megaApi[apiIndex]->setMaxConnections(args..., &rt);
         return rt.waitForResult();
     }
-    /**
-     * @brief Download a file from a URL using cURL
-     *
-     * @param url The URL of the File
-     * @param dstPath The destination file path to write
-     * @return True if the file is downloaded successfully, otherwise false
-     */
-    bool getFileFromURL(const std::string& url, const fs::path& dstPath);
-
-    /**
-     * @brief Download a file from the Artifactory
-     *
-     * @param relativeUrl The relative URL to the base URL
-                          "https://artifactory.developers.mega.co.nz:443/artifactory/sdk/"
-     * @param dstPath The destination file path to write
-     * @return True if the file is downloaded successfully, otherwise false
-     */
-    bool getFileFromArtifactory(const std::string& relativeUrl, const fs::path& dstPath);
 
     /* MegaVpnCredentials */
     template<typename... requestArgs>
@@ -1078,4 +1288,380 @@ public:
 
         return tracker.waitForResult();
     }
+
+    // Checks if a function is a valid invitation list accessor function.
+    template<typename Function>
+    static constexpr auto IsInvitationListAccessorV =
+        std::is_invocable_r_v<const MegaContactRequestList*, Function, const MegaApi*>;
+
+    // Checks if a function is a valid invitation predicate function.
+    template<typename Predicate>
+    static constexpr auto IsInvitationPredicateV =
+        std::is_invocable_r_v<bool, Predicate, const MegaContactRequest&>;
+
+    /**
+     * Accept the specified contact invitation.
+     *
+     * @param client
+     * The client that should accept the contact request.
+     *
+     * @param invitation
+     * The invitation the client should accept.
+     *
+     * @return
+     * API_OK if client could accept the invitation.
+     */
+    Error acceptInvitation(MegaApi& client, const MegaContactRequest& invitation);
+
+    /*
+     * Establish a friendship between two clients.
+     *
+     * @return
+     * API_OK if the friendship could be established.
+     */
+    Error befriend(MegaApi& client0, MegaApi& client1);
+
+    /**
+     * Search a list of invitations for an invitation satisfying some predicate.
+     *
+     * @param invitations
+     * The list of invitations to search.
+     *
+     * @param invitationOK
+     * The predicate an invitation must satisfy to be returned.
+     *
+     * @return
+     * The first invitation satisfying predicate, if any.
+     */
+    template<typename InvitationPredicate>
+    auto findInvitation(const MegaContactRequestList& invitations, InvitationPredicate invitationOK)
+        -> std::enable_if_t<IsInvitationPredicateV<InvitationPredicate>,
+                            std::unique_ptr<MegaContactRequest>>
+    {
+        // Try and find an invitation satisfying our predicate.
+        for (auto i = 0, j = invitations.size(); i < j; ++i)
+        {
+            // Found a suitable invitation!
+            if (auto* invitation = invitations.get(i); std::invoke(invitationOK, *invitation))
+            {
+                return makeUniqueFrom(invitation->copy());
+            }
+        }
+
+        // Couldn't find a suitable invitation.
+        return nullptr;
+    }
+
+    /**
+     * Search a list of invitations for an invitation satisfying some predicate.
+     *
+     * @param getInvitations
+     * An accessor specifying which invitation list we want to search.
+     *
+     * @param invitationOK
+     * The predicate an invitation must satisfy to be returned.
+     *
+     * @return
+     * The first invitation satisfying predicate, if any.
+     */
+    template<typename InvitationListAccessor, typename InvitationPredicate>
+    auto findInvitation(MegaApi& client,
+                        InvitationListAccessor getInvitations,
+                        InvitationPredicate invitationOK)
+        -> std::enable_if_t<IsInvitationListAccessorV<InvitationListAccessor> &&
+                                IsInvitationPredicateV<InvitationPredicate>,
+                            std::unique_ptr<MegaContactRequest>>
+    {
+        // Get invitation list.
+        auto* invitations = std::invoke(getInvitations, &client);
+
+        // No invitations.
+        if (!invitations || invitations->size() == 0)
+        {
+            return nullptr;
+        }
+
+        // Try and find an invitation satisfying our predicate.
+        return findInvitation(*invitations, std::move(invitationOK));
+    }
+
+    /**
+     * Check if email is a contact.
+     *
+     * @param client
+     * The client (user) whose contacts we will search.
+     *
+     * @param email
+     * The contact we are searching for.
+     */
+    auto hasContact(MegaApi& client, const std::string& email) -> std::unique_ptr<MegaUser>;
+
+    /**
+     * Check if a client has received a contact invite from some user.
+     *
+     * @apram client
+     * The client that should've received an invitation.
+     *
+     * @param email
+     * The user that should've sent us the invitation.
+     *
+     * @return
+     * A matching invitation, if found.
+     */
+    auto hasReceivedInvitationFrom(MegaApi& client, const std::string& email)
+        -> std::unique_ptr<MegaContactRequest>;
+
+    /*
+     * Check if this client has sent a contact invite to some user.
+     *
+     * @param client
+     * The client that should've sent the invitation.
+     *
+     * @param email
+     * The user that we've sent the invitation to.
+     *
+     * @return
+     * A matching invitation, if found.
+     */
+    auto hasSentInvitationTo(MegaApi& client, const std::string& email)
+        -> std::unique_ptr<MegaContactRequest>;
+
+    /**
+     * Remove a contact.
+     *
+     * @param client
+     * The client that wants to remove this contact.
+     *
+     * @param email
+     * The contact we want to remove.
+     *
+     * @returns
+     * API_OK if the contact was removed.
+     */
+    Error removeContact(MegaApi& client, const std::string& email);
+
+    /**
+     * Break a contact relationship between two clients.
+     *
+     * @return
+     * API_ENOENT if no contact relationship exists.
+     * API_OK if the contact relationship was successfully broken.
+     */
+    Error removeContact(MegaApi& client0, MegaApi& client1);
+
+    /**
+     * Send a contact invitation from one user to another.
+     *
+     * @param client
+     * The client that should send the invitation.
+     *
+     * @param email
+     * The user that we are inviting to be a contact.
+     *
+     * @return
+     * API_OK if the invitation was sent successfully.
+     */
+    Error sendInvitationTo(MegaApi& client, const std::string& email);
+
+    /**
+     * Send a contact invitation from one client to another.
+     *
+     * @param client0
+     * The client that will be sending the invitation.
+     *
+     * @param client1
+     * The client that will be receiving the invitation.
+     *
+     * @return
+     * {invitation, API_OK} on success.
+     * {nullptr, ?} on failure.
+     */
+    using SendInvitationToResult = std::pair<std::unique_ptr<MegaContactRequest>, Error>;
+
+    auto sendInvitationTo(MegaApi& client0, MegaApi& client1) -> SendInvitationToResult;
+
+    /**
+     * @brief Returns the current tests suite and case name.
+     * @example calling this inside `TEST_F(SdkTest, Foo)` returns `std::pair {"SdkTest", "Foo"}`
+     */
+    std::pair<std::string, std::string> getTestSuiteAndName() const;
+
+    /**
+     * @brief Returns an identifier of the current test. It can be used to prefix log messages.
+     * @example calling this inside `TEST_F(SdkTest, Foo)` returns `"SdkTest.Foo : "`
+     */
+    std::string getLogPrefix() const;
+
+    /**
+     * @brief Gets a file name prefix unique for the test case
+     * @example calling this inside `TEST_F(SdkTest, Foo)` returns `"SdkTest_Foo_"`
+     */
+    std::string getFilePrefix() const;
 };
+
+/**
+ * @brief
+ * Return an object that will restore client's plan on destruction.
+ *
+ * @param client
+ * The client whose plan we want to restore.
+ *
+ * @return
+ * An object that will restore client's plan on destruction.
+ */
+auto accountLevelRestorer(MegaApi& client) -> ScopedDestructor;
+
+/**
+ * @brief
+ * Create a directory with a given name under a specified parent.
+ *
+ * @param client
+ * The client who should create the directory.
+ *
+ * @param parent
+ * The directory where our new directory will live.
+ *
+ * @param name
+ * The name of the directory we are creating.
+ *
+ * @return
+ * An Error if the directory couldn't be created.
+ * An std::unique_ptr<MegaNode> if the directory was created.
+ */
+auto createDirectory(MegaApi& client, const MegaNode& parent, const std::string& name)
+    -> Expected<std::unique_ptr<MegaNode>>;
+
+/**
+ * @brief
+ * Elevate client to a pro pricing plan.
+ *
+ * @param client
+ * The client who we want to elevate to a pro pricing plan.
+ *
+ * @return
+ * An account level restorer on success.
+ * An error on failure.
+ */
+auto elevateToPro(MegaApi& client) -> Expected<ScopedDestructor>;
+
+/**
+ * @brief
+ * Export the specified node.
+ *
+ * @param client
+ * The client that should export the node.
+ *
+ * @param node
+ * The node we want to export.
+ *
+ * @param expirationDate
+ * When should the node's resulting public link expire?
+ *
+ * @return
+ * A string (the node's public link) if the node could be exported
+ * An error if the node couldn't be exported.
+ */
+auto exportNode(MegaApi& client,
+                const MegaNode& node,
+                std::optional<std::int64_t> expirationDate = std::nullopt) -> Expected<std::string>;
+
+/**
+ * @brief
+ * Retrieve a client's account details.
+ *
+ * @param client
+ * The client whose account details we want to retrieve.
+ *
+ * @return
+ * A pointer to an account details instance on success.
+ * An error on failure.
+ */
+auto getAccountDetails(MegaApi& client) -> Expected<std::unique_ptr<MegaAccountDetails>>;
+
+/**
+ * Describes a client's current account level.
+ */
+struct AccountLevel
+{
+    AccountLevel(int months, int plan):
+        months(months),
+        plan(plan)
+    {}
+
+    // How long the client's current pricing is active.
+    int months;
+
+    // The client's current pricing plan.
+    int plan;
+}; // AccountLevel
+
+/**
+ * @brief
+ * Retrieve a client's current account level.
+ *
+ * @param client
+ * The client whose account level we want to retrieve.
+ *
+ * @return
+ * An account level object instance on success.
+ * An error on failure.
+ */
+auto getAccountLevel(MegaApi& client) -> Expected<AccountLevel>;
+
+/**
+ * @brief
+ * Retrieve available pricing plans.
+ *
+ * @param client
+ * The client who should request the available pricing plans.
+ *
+ * @return
+ * A pointer to a pricing object instance on success.
+ * An error on failure.
+ */
+auto getPricing(MegaApi& client) -> Expected<std::unique_ptr<MegaPricing>>;
+
+/**
+ * Import a node into this account via public link.
+ *
+ * @param client
+ * The client who is importing the node.
+ *
+ * @param link
+ * The public link of the node we want to import.
+ *
+ * @param parent
+ * Where the node should be imported.
+ *
+ * @return
+ * An Error if the node could not be imported.
+ * An std::unique_ptr<MegaNode> if the node was imported.
+ */
+auto importNode(MegaApi& client, const std::string& link, const MegaNode& parent)
+    -> Expected<std::unique_ptr<MegaNode>>;
+
+/**
+ * @brief
+ * Set a client's account level.
+ *
+ * @param client
+ * The client whose account level we want to set.
+ *
+ * @param args
+ * Arguments suitable for MegaApi::sendSetAccountLevelDevCommand(...).
+ *
+ * @return
+ * API_OK on success.
+ */
+template<typename... requestArgs>
+Error setAccountLevel(MegaApi& client, requestArgs... args)
+{
+    // So we can wait for the client's result.
+    RequestTracker tracker(&client);
+
+    // Try and set the user's account level.
+    client.sendSetAccountLevelDevCommand(args..., &tracker);
+
+    // Return client's result to caller.
+    return tracker.waitForResult();
+}
