@@ -14,10 +14,16 @@ protected:
         bool wait; // wait for response
     };
 
-    void createShareAtoB(MegaNode* node, const Party& partyA, const Party& partyB);
+    void createShareAtoB(MegaNode* node,
+                         const Party& partyA,
+                         const Party& partyB,
+                         int accessType = MegaShare::ACCESS_READWRITE);
 
     // Use mApi[0] as party A and mApi[1] as party B
-    void createShareAtoB(MegaNode* node, bool waitForA = true, bool waitForB = true);
+    void createShareAtoB(MegaNode* node,
+                         bool waitForA = true,
+                         bool waitForB = true,
+                         int accessType = MegaShare::ACCESS_READWRITE);
 
     // Remove a share ensuring node changes are notified.
     // Use mApi[0] and mApi[1]
@@ -69,7 +75,10 @@ protected:
     bool waitForOutShare(MegaApi& api, unsigned expectedNum);
 };
 
-void SdkTestShare::createShareAtoB(MegaNode* node, const Party& partyA, const Party& partyB)
+void SdkTestShare::createShareAtoB(MegaNode* node,
+                                   const Party& partyA,
+                                   const Party& partyB,
+                                   int accessType)
 {
     assert(partyA.apiIndex < mApi.size());
     assert(partyB.apiIndex < mApi.size());
@@ -84,8 +93,7 @@ void SdkTestShare::createShareAtoB(MegaNode* node, const Party& partyA, const Pa
     apiB.mOnNodesUpdateCompletion = createOnNodesUpdateLambda(node->getHandle(),
                                                               MegaNode::CHANGE_TYPE_INSHARE,
                                                               apiB.nodeUpdated);
-    ASSERT_NO_FATAL_FAILURE(
-        shareFolder(node, apiB.email.c_str(), MegaShare::ACCESS_READWRITE, partyA.apiIndex));
+    ASSERT_NO_FATAL_FAILURE(shareFolder(node, apiB.email.c_str(), accessType, partyA.apiIndex));
 
     if (partyA.wait)
     {
@@ -103,9 +111,9 @@ void SdkTestShare::createShareAtoB(MegaNode* node, const Party& partyA, const Pa
     apiA.nodeUpdated = apiB.nodeUpdated = false;
 }
 
-void SdkTestShare::createShareAtoB(MegaNode* node, bool waitForA, bool waitForB)
+void SdkTestShare::createShareAtoB(MegaNode* node, bool waitForA, bool waitForB, int accessType)
 {
-    createShareAtoB(node, {0, waitForA}, {1, waitForB});
+    createShareAtoB(node, {0, waitForA}, {1, waitForB}, accessType);
 }
 
 void SdkTestShare::removeShareAtoB(MegaNode* node)
@@ -153,8 +161,9 @@ void SdkTestShare::addContactsAndVerifyCredential(unsigned fromApiIndex, unsigne
     ASSERT_NO_FATAL_FAILURE(getContactRequest(toApiIndex, false));
 
     mApi[fromApiIndex].contactRequestUpdated = mApi[toApiIndex].contactRequestUpdated = false;
-    ASSERT_NO_FATAL_FAILURE(
-        replyContact(mApi[toApiIndex].cr.get(), MegaContactRequest::REPLY_ACTION_ACCEPT));
+    ASSERT_NO_FATAL_FAILURE(replyContact(mApi[toApiIndex].cr.get(),
+                                         MegaContactRequest::REPLY_ACTION_ACCEPT,
+                                         toApiIndex));
     ASSERT_TRUE(waitForResponse(&mApi[toApiIndex].contactRequestUpdated))
         << "Accepting contact timeout: " << maxTimeout << " seconds";
     ASSERT_TRUE(waitForResponse(&mApi[fromApiIndex].contactRequestUpdated))
@@ -1137,3 +1146,58 @@ TEST_F(SdkTestShareOrder, GetOutSharesOrUnverifiedOutSharesOrderedByCreationTime
     };
     ASSERT_THAT(shares, testing::ElementsAreArray(expectedShares));
 }
+
+TEST_F(SdkTestShare, TestSharesPermission)
+{
+    LOG_info << "___TEST TestSharesPermission___";
+
+    ASSERT_NO_FATAL_FAILURE(getAccountsForTest(3));
+
+    // Ensure no account has the other verified from previous unfinished tests.
+    ASSERT_NO_FATAL_FAILURE(resetCredential(0, 1));
+    ASSERT_NO_FATAL_FAILURE(resetCredential(0, 2));
+
+    LOG_info << "Invite from account 0 to 1 and verify credential";
+    ASSERT_NO_FATAL_FAILURE(addContactsAndVerifyCredential(0, 1));
+    LOG_info << "Invite from account 0 to 2 and verify credential";
+    ASSERT_NO_FATAL_FAILURE(addContactsAndVerifyCredential(0, 2));
+
+    // Root node
+    std::unique_ptr<MegaNode> remoteRootNode(megaApi[0]->getRootNode());
+    ASSERT_NE(remoteRootNode.get(), nullptr);
+
+    LOG_info << "Create folders";
+    auto [handle1, shareNode1] = createFolder(0, "share1", remoteRootNode.get());
+    auto [handle2, shareNode2] = createFolder(0, "share2", remoteRootNode.get());
+    ASSERT_THAT(shareNode1, testing::NotNull());
+    ASSERT_THAT(shareNode2, testing::NotNull());
+
+    LOG_info << "Share (full access) folder from account 0 to account 1 share node 1";
+    ASSERT_NO_FATAL_FAILURE(createShareAtoB(shareNode1.get(), true, true, MegaShare::ACCESS_FULL));
+    {
+        string sharedPath = megaApi[0]->getMyEmail();
+        sharedPath.append(":share1");
+        RequestTracker listener{megaApi[1].get()};
+        auto sharedNode = megaApi[1]->getNodeByPath(sharedPath.c_str());
+        megaApi[1]->setNodeLabel(sharedNode, 1, &listener);
+        ASSERT_TRUE(API_OK == listener.waitForResult());
+        ASSERT_TRUE(WaitFor(
+            [this]() -> bool
+            {
+                auto sharedNode = megaApi[0]->getNodeByPath("/share1");
+                return (1 == sharedNode->getLabel());
+            },
+            20 * 1000));
+    }
+
+    LOG_info << "Share (Read and Write) folder from account 0 to account 2 share node 2";
+    ASSERT_NO_FATAL_FAILURE(createShareAtoB(shareNode2.get(), {0, true}, {2, true}));
+    {
+        string sharedPath = megaApi[0]->getMyEmail();
+        RequestTracker listener{megaApi[2].get()};
+        sharedPath.append(":share2");
+        auto sharedNode = megaApi[2]->getNodeByPath(sharedPath.c_str());
+        megaApi[2]->setNodeLabel(sharedNode, 1, &listener);
+        ASSERT_TRUE(API_EACCESS == listener.waitForResult());
+    }
+}
