@@ -31,7 +31,7 @@ class FsNode;
 
 mega::handle nextFsId();
 
-std::shared_ptr<mega::MegaClient> makeClient(mega::MegaApp& app);
+std::shared_ptr<mega::MegaClient> makeClient(mega::MegaApp& app, mega::DbAccess* dbAccess  = nullptr);
 
 mega::Node& makeNode(mega::MegaClient& client, mega::nodetype_t type, mega::NodeHandle handle, mega::Node* parent = nullptr);
 
